/*
  $Id: color.c,v 1.2 2001/03/01 02:10:08 hvr Exp $

  Copyright (C) 2000 Herbert Valerio Riedel <hvr@gnu.org>

  Color codes were stolen from somewhere...

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

static const char _rcsid[] = "$Id: color.c,v 1.2 2001/03/01 02:10:08 hvr Exp $";

#include "color.h"

const gchar *
color_lookup (gchar code)
{
  switch (code) 
    { 
    case '0' : return "\x1B[0m"; break;
      /*  a standard ! */
    case '!' : return "!"; break;
      /* blinking */
    case 'X' : return "\x1B[5m"; break;

      /*********************/
      /* FOREGROUND COLORS */
      /*********************/
      /* FG black */
    case 'a' : return "\x1B[0;30m"; break;
      /* FG red */
    case 'b' : return "\x1B[0;31m"; break;
      /* FG green */
    case 'c' : return "\x1B[0;32m"; break;
      /* FG brown */
    case 'd' : return "\x1B[0;33m"; break;
      /* FG blue */
    case 'e' : return "\x1B[0;34m"; break;
      /* FG magenta */
    case 'f' : return "\x1B[0;35m"; break;
      /* FG cyan */
    case 'g' : return "\x1B[0;36m"; break;
      /* FG light gray */
    case 'h' : return "\x1B[0;37m"; break;
      /* FG dark gray */
    case 'A' : return "\x1B[1;30m"; break;
      /* FG light red */
    case 'B' : return "\x1B[1;31m"; break;
      /* FG light green */
    case 'C' : return "\x1B[1;32m"; break;
      /* FG yellow */
    case 'D' : return "\x1B[1;33m"; break;
      /* FG light blue */
    case 'E' : return "\x1B[1;34m"; break;
      /* FG light magenta */
    case 'F' : return "\x1B[1;35m"; break;
      /* FG light cyan */
    case 'G' : return "\x1B[1;36m"; break;
      /* FG white */
    case 'H' : return "\x1B[1;37m"; break;


      /*********************/
      /* BACKGROUND COLORS */
      /*********************/
      /* BG black */
    case '1' : return "\x1B[1;40m"; break;
      /* BG red */
    case '2' : return "\x1B[1;41m"; break;
      /* BG green */
    case '3' : return "\x1B[1;42m"; break;
      /* BG brown */
    case '4' : return "\x1B[1;43m"; break;
      /* BG blue */
    case '5' : return "\x1B[1;44m"; break;
      /* BG magenta */
    case '6' : return "\x1B[1;45m"; break;
      /* BG cyan */
    case '7' : return "\x1B[1;46m"; break;
      /* BG white */
    case '8' : return "\x1B[1;47m"; break;

    default: return NULL; break;
    }  

  g_assert_not_reached ();
}

gchar *
color_expand (const gchar str[], gboolean strip_color)
{
  const gchar *p;
  GString *tmp;
  gchar *result = NULL;

  g_return_val_if_fail (str != NULL, NULL);

  p = str;
  tmp = g_string_new (NULL);

  while (*p)
    {
      const gchar *expanded_color;

      if (*p != '!' || *(p+1) == '\0' ) /* /([^!]|!$)/ */
        g_string_append_c (tmp, *p);
      else if (*(p+1) == '!')           /* /(!!)/ */
        g_string_append_c (tmp, *p++);
      else if ((expanded_color = color_lookup (p[1]))) 
	p++, g_string_append (tmp, strip_color ? "" : expanded_color);
      else
	g_string_append_c (tmp, *p);
      
      p++;
    }
  
  result = tmp->str;
  g_string_free (tmp, FALSE);

  return result;
}
