/*
    $Id: userdb.c,v 1.3 2001/07/06 10:21:35 prime Exp $

    Copyright (C) 2000 Herbert Valerio Riedel <hvr@gnu.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

static const char _rcsid[] = "$Id: userdb.c,v 1.3 2001/07/06 10:21:35 prime Exp $";

#include <glib.h>
#include <string.h>
#include "userdb.h"

GHashTable *__users = NULL;
GHashTable *__groups = NULL;

const gchar * 
userdb_username_by_uid (guint32 uid)
{
  g_return_val_if_fail (__users != NULL, NULL);

  return g_hash_table_lookup (__users, GUINT_TO_POINTER (uid));
}

const gchar *
userdb_groupname_by_gid (guint32 gid)
{
  g_return_val_if_fail (__groups != NULL, NULL);

  return g_hash_table_lookup (__groups, GUINT_TO_POINTER (gid));
}

void
userdb_register_gid (guint32 gid, const gchar groupname[])
{
  g_return_if_fail (groupname != NULL);

  if (!__groups)
    __groups = g_hash_table_new (NULL, NULL);

  /* fixme g_free() lookup */

  g_hash_table_insert (__groups, GUINT_TO_POINTER (gid), g_strdup (groupname));
}

void userdb_register_uid (guint32 uid, const gchar username[])
{
  g_return_if_fail (username != NULL);

  if (!__users)
    __users = g_hash_table_new (NULL, NULL);

  /* fixme g_free() lookup */
  
  g_hash_table_insert (__users, GUINT_TO_POINTER (uid), g_strdup (username));
}

static guint32 __uid = 0; /* primary uid */
static guint32 __gid = 0; /* primary gid */
static guint   __ngids = 0; /* no. of 2ndary gids */
static guint32 *__gids = NULL; /* 2ndary gids */

void
userdb_setuser (guint32 uid, guint32 gid, guint ngids, const guint32 gids[])
{
  gsize len = sizeof (guint32) * ngids;

  __uid = uid;
  __gid = gid;
  __ngids = ngids;
  __gids = g_realloc (__gids, len);
  memcpy (__gids, gids, len);
}

gboolean 
userdb_have_group (guint32 gid)
{
  gint idx;

  if (gid == __gid)
    return TRUE;

  for (idx = 0; idx < __ngids; idx++)
    if (gid == __gids[idx])
      return TRUE;

  return FALSE;
}

guint32 userdb_get_gid ()
{
  return __gid;
}

guint32 userdb_get_uid ()
{
  return __uid;
}
