/*
  $Id: control.c,v 1.3.2.4 2002/10/26 09:55:18 prime Exp $

  Copyright (C) 2000 Herbert Valerio Riedel <hvr@gnu.org>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <errno.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include "externs.h"
#include "control.h"
#include "color.h"

#ifdef TLS
#define FPUTS tls_fputs
#else
#define FPUTS fputs
#endif

/* temporary local wrapper 
 * to be replaced with new event concept to come... (hvr)
 */
static void ctl_channel_puts (const gchar str[]) 
{
    if (FPUTS (str, sockf) == EOF)
    {
	g_printerr ("fputs (): %s\n", g_strerror (errno));
    }
}


/* exported functions */
gint send_reply_full (gint code, gboolean terminate, gboolean expand_color, 
    gboolean leading_space, const gchar reply[]) 
{
    gchar **lines = NULL;  
    gint i        = 0;
    gchar *tmp    = NULL;

    /* return -1 if the reply code is invalid */
    g_return_val_if_fail(code >= 100 && code <= 599, -1);
    g_return_val_if_fail(reply != NULL, -1);

    /* almost unworthy optimization: don't waste effort on expanding colors if 
       there ain't a '!' in the reply string */

    if (expand_color && !strchr(reply, '!')) expand_color = FALSE;
    if (expand_color)
    {
	gchar* _reply = color_expand (reply, !(usr.class & UF_COLOR));
	lines = g_strsplit (_reply, "\n", 0);
	g_free (_reply);
    }
    else lines = g_strsplit (reply, "\n", 0);

  /*
    for (i = 0; lines[i]; i++) {
    gboolean last = (lines[i+1] == NULL) && terminate;
    gchar *tmp = NULL;
    g_strchomp (lines[i]);
    tmp = g_strdup_printf ("%d%c%s\r\n", code, 
    last ? ' ' : '-', lines[i]);
    ctl_channel_puts (tmp);
    g_free (tmp);
    }
    g_strfreev (lines);
  */

    for (i = 0; lines[i]; i++) 
    {
	gboolean last = (lines[i+1] == NULL) && terminate;
	gchar *tmp = NULL;
	g_strchomp (lines[i]);
	/* the last line */
	if (i == 0 || last) 
	{
	    tmp = g_strdup_printf ("%d%c%s\r\n", code, last ? ' ' : '-', lines[i]);
	}
	/* a normal line */
	else 
	{
	    if (leading_space)
	    {
		tmp = g_strdup_printf (" %s\r\n", lines[i]);
	    }
	    else 
	    {
		tmp = g_strdup_printf ("%s\r\n", lines[i]);	    
	    }
	}
	ctl_channel_puts (tmp);
	g_free (tmp);
    }
    g_strfreev (lines);
    return 0;
}


/* Send reply with variable arguments and a format string */
gint send_reply_full_f (gint code, gboolean terminate, gboolean expand_color,
    const gchar fmt[], ...)
{
    va_list args;
    gchar *printed = NULL;
    gint retval = 0;

    g_return_val_if_fail (fmt != NULL, -1);

    va_start (args, fmt);
    printed = g_strdup_vprintf (fmt, args);
    va_end (args);

    g_assert (printed != NULL);
    retval = send_reply_full(code, terminate, expand_color, FALSE, printed);
    g_free (printed);
   
    return retval;
}
