/*   $Id: ftps.c,v 1.29.2.10 2003/04/10 17:44:41 elmoe Exp $
 *
 *   This file is part of OpenFTPD
 *
 *   Copyright (C) 1996-1999 by Crescent
 *   Copyright (C) 2001 by HVR
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "ac_config.h"

#include <unistd.h>
#include <sys/types.h>
#include <sys/param.h>
#include <errno.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/file.h>

#include "globals.h"
#include "common.h"
#include "log.h"
#include "perm.h"
#include "perl_layer.h"

#include "rfc931.h"
#include "utils.h"
#include "site_vfs.h"

#include "cmd_protos.h"
#include "strlcpy.h"
#include "strlcat.h"

#ifdef TLS
#include "tlsutil.h"
#endif

void
quit(int reason)
{
  close_server++;
  if (close_reason == CL_ERROR)
    close_reason = reason;
  if ((reason == CL_NORMAL) || (reason == CL_SOCKERR))
    close_server++;
}

void
signal_handler (int sig)
{
  if (sig == SIGTERM)
    exit (CL_KILL);
  if (((sig_write == sig_size - 1) && !sig_read)
      || (sig_write + 1 == sig_read))
    return;
  sig_array[sig_write] = sig;
  if (++sig_write >= sig_size)
    sig_write = 0;
}

void
handle_signals (void)
{
  while (sig_read != sig_write)
    {
      switch (sig_array[sig_read])
	{
	case SIGHUP:
	  quit (CL_SHUTDOWN);
	  break;
	case SIGINT:
	  quit (CL_CLOSE);
	  break;
	case SIGQUIT:
	  quit (CL_KICK);
	  break;
	case SIGPIPE:
	  quit (CL_PIPEERR);
	  break;
	case SIGALRM:
	  quit (CL_TIMEOUT);
	  break;
	case SIGUSR1:
	  close_reason = CL_ERROR;
	  close_server = 0;
	  break;
	}
      if (++sig_read >= sig_size)
	sig_read = 0;
    }
}

int
cmd_auth(char **p,
         int n,
         char *buf,
         int buflen)
{
#ifdef TLS
  char *method, *str;

  if (n < 1)
    method = strdup("UNKNOWN");
  else
    method = strdup(p[0]);

  if (!method)
    exit(1);

  if (!strcasecmp(method, "TLS") || !strcasecmp(method, "TLS-C")) {
    send_reply(234, "AUTH TLS successfull.");

    if (tls_accept_ctrl(sock)) {
      send_reply(431, "TLS negotiation on control channel failed.");
      log_action(0, LG_SECURITY, "AUTH",
                 "by %s FAILED: TLS negotiation on control channel.",
                 usr.name);
    } else
      tls_on_ctrl = 1;
  } else {
    str =
      (char *) malloc(sizeof(char) *
                      (strlen("AUTH:  method unsupported.") + strlen(method) +
                       1));

    if (!str)
      exit(1);

    snprintf(str, sizeof(char) * (strlen("AUTH:  method unsupported.") + strlen(method) + 1),
             "AUTH: %s method unsupported.", method);
    send_reply(504, str);
  }

  return 0;
#else
  return send_reply(502, "AUTH: command unrecognized.");
#endif
}

int cmd_pbsz(char **p, int n, char *buf, int buflen) {
#ifdef TLS
  if (!tls_on_ctrl)
    return send_reply(503, "You are currently using an insecure control connection.");
  else if ((n == 1) && (atoi(p[0]) == 0))
    send_reply(200, "PBSZ 0 successfull.");
  else
    send_reply(200, "PBSZ=0 successfull.");
  
  tls_pbsz_ok = 1;
  
  return 0;
#else
  return send_reply(502, "PBSZ: command unrecognized.");
#endif
}

int cmd_prot(char **p, int n, char *buf, int buflen) {
#ifdef TLS
  char *str;

  if (!tls_pbsz_ok)
    return send_reply(503, "You must issue a PBSZ command prior to calling PROT.");
  
  if (n == 1) {
    switch (p[0][0]) {
      case 'C':
        send_reply(200, "Protection set to clear.");
        tls_on_data = 0;
 
        break;
      case 'P':
        send_reply(200, "Protection set to private.");
        tls_on_data = 1;

        break;
      case 'S':
        send_reply(536, "PROT method S unsupported.");

        break;
      case 'E':
        send_reply(536, "PROT method E unsupported.");
 
        break;
      default:
        str = (char *) malloc(sizeof(char) * (strlen("PROT method  unsupported.") + strlen(p[0]) + 1));

        snprintf(str, sizeof(char) * (strlen("PROT method  unsupported.") + strlen(p[0]) + 1), 
          "PROT method %s unsupported.", p[0]);
        send_reply(504, str);
    }
  } else 
    send_reply(504, "PROT method unsupported.");
  
  return 0;
#else
  return send_reply(502, "PROT: command unrecognized.");
#endif
}

/* new function for handling ctl_connection input... 
 * will be integrated soon (hvr)
 */
static void
handle_ftp_command(const gchar str[])
  G_GNUC_UNUSED /* yet */ ;

  static void
    dispatch_ftp_command_new(const gchar cmd[],
                             const gchar arg[])
{
  struct {
    const gchar *command;
      gint(*function) (gchar **,
                       gint,
                       gchar *,
                       gint);
    gboolean must_be_loggedin;
    gboolean parse_command;
  } ftpcmds[] = {
    {
    0, 0, 0, 0}
    };

  gint i;

  g_printerr("%s: cmd='%s' arg='%s'\n", G_GNUC_PRETTY_FUNCTION, cmd, arg);

  if (!cmd) {
    send_reply_raw(500, "no command given.");
      return;
    }

  for (i = 0; ftpcmds[i].command; i++)
    if (!g_strcasecmp(cmd, ftpcmds[i].command))
      break;

  if (!ftpcmds[i].command) {	/* cmd not found */
    send_reply_raw(500, "Unrecognised command.");
      return;
    }
  
  if (!logged_in && ftpcmds[i].must_be_loggedin) {
    send_reply_raw(530, "Please login with USER and PASS.");
      return;
    }

  if (arg)
    ftpcmds[i].function((gchar **) & arg, 1, NULL, 0);
  else
    ftpcmds[i].function(NULL, 0, NULL, 0);
}

void
handle_ftp_command(const gchar str[])
{
  const gchar *c = str;
  const gchar *c2;
  gchar *cmd = NULL, *arg = NULL;

  while (*c == ' ')
    c++;
  
  c2 = strchr (c, ' ');

  if (!*c)
    c = NULL;

  /* c is at beginning of cmd, or NULL if no cmd
     c2 is at beginning of arg, or NULL if no arg */
  if (!c)
    cmd = arg = NULL;
  else if (!c2)
    cmd = g_strdup (c), arg = NULL;
  else
    cmd = g_strndup (c, c2 - c), arg = g_strdup (c2 + 1);

  dispatch_ftp_command_new (cmd, arg);

  g_free (cmd);
  g_free (arg);
}



int
dispatch_command (char *buffer, int bufferlen, char **p, int n)
{
  /* fixme hvr -- hash it */
  struct
  {
    const gchar *command;
    gint (*function) (gchar **, gint, gchar *, gint);
    gboolean must_be_loggedin;
    gboolean parse_command;
  } 
  ftpcmd[] =
    { /* fixme hvr -- get rid of parse_command field */
      /* replace must_be_loggedin by some better state machine concept */
      { "ABOR", cmd_abor, TRUE,  FALSE },
      { "ACCT", cmd_acct, TRUE,  FALSE },
      { "ALLO", cmd_allo, TRUE,  FALSE }, 
      { "APPE", cmd_appe, TRUE,  FALSE },
      { "AUTH", cmd_auth, FALSE, FALSE },
      { "CDUP", cmd_cdup, TRUE,  FALSE },
      { "CWD",  cmd_cwd,  TRUE,  FALSE },
      { "DELE", cmd_dele, TRUE,  FALSE },
      { "HELP", cmd_help, TRUE,  FALSE },
      { "LIST", cmd_list, TRUE,  TRUE },
      { "MDTM", cmd_mdtm, TRUE,  FALSE },
      { "MKD",  cmd_mkd,  TRUE,  FALSE },
      { "MODE", cmd_mode, FALSE, FALSE },
      { "NLST", cmd_nlst, TRUE,  TRUE },
      { "NOOP", cmd_noop, TRUE,  FALSE },
      { "PASS", cmd_pass, TRUE,  FALSE },
      { "PASV", cmd_pasv, TRUE,  FALSE },
      { "PBSZ", cmd_pbsz, FALSE, FALSE },
      { "PORT", cmd_port, TRUE,  FALSE },
      { "PROT", cmd_prot, FALSE, FALSE },
      { "PWD",  cmd_pwd,  TRUE,  FALSE },
      { "QUIT", cmd_quit, FALSE, FALSE },
      { "REIN", cmd_rein, TRUE,  FALSE },
      { "REST", cmd_rest, TRUE,  FALSE },
      { "RETR", cmd_retr, TRUE,  FALSE },
      { "RMD",  cmd_rmd,  TRUE,  FALSE },
      { "RNFR", cmd_rnfr, TRUE,  FALSE },
      { "RNTO", cmd_rnto, TRUE,  FALSE },
      { "SITE", cmd_site, TRUE,  FALSE },
      { "SIZE", cmd_size, TRUE,  FALSE },
      { "SMNT", cmd_smnt, TRUE,  FALSE },
      { "STAT", cmd_stat, TRUE,  FALSE },
      { "STOR", cmd_stor, TRUE,  FALSE },
      { "STOU", cmd_stou, TRUE,  FALSE },
      { "STRU", cmd_stru, FALSE, FALSE },
      { "SYST", cmd_syst, FALSE, FALSE },
      { "TYPE", cmd_type, FALSE, TRUE },
      { "USER", cmd_user, FALSE, FALSE },
      { "XMKD", cmd_mkd,  TRUE,  FALSE },
      { "XPWD", cmd_pwd,  TRUE,  FALSE },
      { 0, 0, 0, 0 }
    };

  gint j;

  for (j = 0; ftpcmd[j].command; j++)
    if (!g_strcasecmp (ftpcmd[j].command, *p))
      break;

  if (!ftpcmd[j].command)
    {
      /* not found */
      return send_reply_full_f (500, TRUE, FALSE, "Unrecognised command (%s).", *p);
    }

  if (!logged_in && ftpcmd[j].must_be_loggedin)
    {
      retries++;
      if (login_retries && (retries >= login_retries))
	{
	  quit (CL_NORMAL);
	  log_action (0, LG_SECURITY, "LOGIN",
		      "FAILED: Not logged in yet");
	  return send_reply (421, "Not logged in yet, connection closed.");
	}
      return send_reply (503, "Please login first.");
    }
  
  if ((n == 2) && ftpcmd[j].parse_command)
    {
      n--;
      if (parse_command (*(p + 1), p, 31, &n))
	return send_reply (500, "Too many parameters.");
    }

  return ftpcmd[j].function (p + 1, n - 1, buffer, bufferlen);
}

void get_string(char **buffer, char *str, int max) {
  fgets (str, max, stdin);
  if (*str) {
    if (!*buffer) {
      if(!(*buffer = g_strdup(str))) {
	errlog ("%s %d: get_string(): %s", __FILE__, __LINE__, strerror(errno));
	exit (close_reason);
      }
      g_strchomp(str);
      g_strchomp(*buffer);
    }
    else {
      *buffer = NULL;
    }
  }
}

void prepare_msg (struct msg_s *msg) {
  char *buffer;
  int err;
  FILE *file;
  char filename[256];
  char *name = NULL;
  struct stat st;
  size_t bufferlen;
  int stat_err;

  if(!msg) return;

  /* prepare the filename */
  if (usr.name[0]) {
    g_snprintf(filename, sizeof(filename), "%s%s%s/%s", server_dir, DFL_MSGUDIR, usr.name, msg->filename);
    err = stat (filename, &st);
    stat_err = errno;

    if (err) {
      if((name = get_groupname (usr.gid))) {
	g_snprintf(filename, sizeof(filename), "%s%s%s/%s", server_dir, DFL_MSGGDIR, name, msg->filename);
	err = stat (filename, &st);
	stat_err = errno;
      }
    }
  }
  else
    err = -1;

  if (err) {
    g_snprintf(filename, sizeof(filename), "%s%s%s", server_dir, DFL_MSGDIR, msg->filename);
    err = stat (filename, &st);
    stat_err = errno;
  }

  /* do the actual reading the file */
  /* file exists and has changed since last read */
  if (!err && st.st_mtime != msg->filedate) {
    buffer = msg->msg;
    bufferlen = msg->size;

    /* msg->msg points to NULL. allocate a buffer large enough to hold entire file + newline + NULL */
    if(!buffer) {
      bufferlen = st.st_size + 2 * sizeof(char);
      if((buffer = g_malloc(bufferlen))) {
	msg->msg = buffer;
	msg->size = bufferlen;
      }
    }

    if (buffer) {
      if((file = fopen (filename, "rb"))) {
#ifdef HAVE_FLOCK
	flock(fileno(file), LOCK_SH|LOCK_NB);
#endif
	msg->filedate = st.st_mtime;

	if(st.st_size + 2 * sizeof(char) > msg->size) {
	  bufferlen = msg->size + st.st_size + 2 * sizeof(char);

	  if(!(buffer = g_realloc(buffer, bufferlen))) {
	    /* we can't get a buffer large enough to hold the entire file
	     * we don't want truncation, since we allow execution of scripts from within message files
	     * so load default message and return
	     */
	    errlog("%s %d: g_realloc() failed (%d): %s", __FILE__, __LINE__, errno, strerror(errno));
	    if(msg->msg) g_free(msg);

	    msg->msg = msg->default_msg;
	    msg->size = 0;
	    msg->len = strlen(msg->msg);
	    return;
	  }
	}
	else
	  msg->len = fread (msg->msg, 1, st.st_size, file) / sizeof(char);

	/* terminate msg->msg buffer, since fread will not do that for us */
	*(msg->msg + msg->len) = '\0';

#ifdef HAVE_FLOCK
	flock(fileno(file), LOCK_UN);
#endif
	fclose (file);

	/* remove trailing whitespaces (space, newline, ..) */
	g_strchomp(msg->msg);
	/* append a trailing newline */
	msg->len = strlcat(msg->msg, "\n", msg->size);

	/* there's no room for newline. reallocate a new buffer to handle it
	 * msg->len holds the number of characters strlcat tried to create - 1
	 * so we can use that for the size of the new buffer
	 */
	if(msg->len >= msg->size) {
	  /* if g_realloc fails, load the default message and return */
	  bufferlen = msg->len + sizeof(char);
	  if((buffer = g_realloc(msg->msg, bufferlen))) {
	    msg->size = bufferlen;
	    strlcat(msg->msg, "\n", msg->size);
	  }
	  else {
	    errlog("%s %d: g_realloc() failed (%d): %s", __FILE__, __LINE__, errno, strerror(errno));
	    g_free(msg);
	    msg->msg = msg->default_msg;
	    msg->size = 0;
	    msg->len = strlen(msg->msg);
	    return;
	  }
	}
	msg->len = strlen(msg->msg);
      }
    }
  }

  if (!msg->msg || !msg->len) {
    if(msg->msg)
      g_free(msg->msg);
    msg->msg = msg->default_msg;
    msg->len = strlen (msg->msg);
    msg->size = 0;
  }
}

int
main (int argc, char **argv, char **env)
{
  int bouncer;
  struct sockaddr_in loc_addr;
  int n;
  char *p[32];
  char *q;
#ifndef HAVE_SIGSET
  struct sigaction sig_act;
#endif
  struct sockaddr_in rem_addr;
  char str[256];
  int timeout;
#ifdef TLS
  char buf[256];
#endif

#ifndef HAVE_SIGSET
  sigfillset (&sig_act.sa_mask);
  sig_act.sa_flags = SA_RESTART;
#define sigset(x,y) sig_act.sa_handler = y; sigaction(x,&sig_act,NULL)
#endif
  sigset (SIGHUP, signal_handler);
  sigset (SIGINT, signal_handler);
  sigset (SIGQUIT, signal_handler);
  sigset (SIGALRM, signal_handler);
  sigset (SIGPIPE, signal_handler);
  sigset (SIGTERM, signal_handler);
  sigset (SIGUSR1, signal_handler);
  sigset (SIGUSR2, signal_handler);
  setvbuf (stdout, NULL, _IOLBF, 0);

  /* get some vars from ftpd */
  printf ("SIN\n");
  scanf ("%lu %lu %i %i %i %ld %i %i %i %i %i %i %i ", 
         &other_ip_s, &passive_ip_s,
	 &login_retries, &flags, &showinfo, &ipaddr, &sock, &fconv_mode,
	 &checktimeout, &bouncer, &timeout, &sitegive, &min_freemb);
  get_string (&permission_file, str, sizeof (str));
  get_string (&readme_file, str, sizeof (str));
  get_string (&hidden_files, str, sizeof (str));
  get_string (&dupe_path, str, sizeof (str));
  get_string (&ff_path, str, sizeof (str));
  get_string (&pre_download, str, sizeof (str));
  get_string (&pre_upload, str, sizeof (str));
  get_string (&rejected_hops, str, sizeof (str));
  get_string (&kickonnuke, str, sizeof (str));
  fgets (server_dir, sizeof (server_dir), stdin);
  if (server_dir[strlen (server_dir) - 1] == '\n')
    server_dir[strlen (server_dir) - 1] = '\0';

  if ((sockf = fdopen (sock, "wt")) == NULL
      || (sockfr = fdopen (sock, "rt")) == NULL)
    {
      errlog ("fdopen(): %s (%i)", strerror (errno), errno);
      return CL_SOCKERR;
    }

  setvbuf (sockf, NULL, _IOLBF, 0);

#ifdef TLS
  tls_set_defaults();

  if (tls_init())
    send_reply(530, "TLS subsystem failed to initialize.");

  /* see if the site requires tls protection to login */
  fprintf(stdout, "TLS_PRT\n");

  if (fgets(buf, sizeof(buf), stdin))
    if (!strcasecmp(buf, "+yes\n"))
      tls_force_prot = 1;
#endif

  /* bouncer and ident check 0x01=bouncer, 0x02=ident */
  *usr.ident = '\0';
  usr.ipaddr = 0;
  if (bouncer & 0x01)
    {
      /* bouncer handling, bnc4all & rftp compatible
         bnc4all syntax: AUTH [<user>]@<ip>[ <anything_else_ignored>]
         rftp syntax:    IDNT [<user>]@<ip>:[ <anything_else_ignored>]
       */
      if (fgets_to (str, sizeof (str), sockfr, timeout))
	{
	  *p = strchr (str, '@');
	  q = strchr (*p, ':');
	  if (q)
	    *q = '\0';
	  if (*p)
	    {
	      /* errlog("IP check: %s", (*p)+1); */
	      usr.ipaddr = ntohl (inet_addr ((*p) + 1));
	      **p = '\0';
	      *p = strchr (str, ' ');
	      if (*p)
		{
		  /* errlog("Ident check: %s", (*p)+1); */
		  strlcpy (usr.ident, (*p) + 1, sizeof (usr.ident));
		}
	    }
	}
    }
  if ((bouncer & 0x02) && !*usr.ident)
    {
      n = sizeof (loc_addr);
      if (getsockname (sock, (struct sockaddr *) &loc_addr, &n) < 0)
	{
	  bouncer = 0;
	}
      n = sizeof (rem_addr);
      if (getpeername (sock, (struct sockaddr *) &rem_addr, &n) < 0)
	{
	  bouncer = 0;
	}
      if (bouncer)
	{
	  strlcpy (usr.ident, rfc931 (&rem_addr, &loc_addr),
		   sizeof (usr.ident));
	}
    }
  if (!*usr.ident)
    strlcpy (usr.ident, "UNKNOWN", sizeof (usr.ident));
  if (usr.ipaddr)
    ipaddr = usr.ipaddr;

  /* send ident@ip to ftpd for check, exit if check fails */
  printf ("IDT_CHK %lu %s\n", usr.ipaddr, usr.ident);
  fgets (str, sizeof (str), stdin);
  if (*str != '+')
    {
      quit (CL_IDENT);
      /* this is a stupid abuse of the UUD command, we should add
         a special command for this */
      printf ("UUD %i\n", close_reason);
      fgets (str, sizeof (str), stdin);
      return close_reason;
    }

  {
    gchar sitedir[MAXPATHLEN] = { 0, };

    if (!getcwd (sitedir, sizeof (sitedir)))
      {
	send_reply (421, "failed to get wd");
	quit (CL_ERROR);
	return close_reason;
      }

    vfs_init (sitedir);
  }

  /* send startup message and enter command loop */
  perl_init ();

  {				/* fixme_hvr -- make it less hacky :-) */
    char tmp[1024] = { 0, };

    /* initialize important vars */
    perl_set_var_str ("FTP::cfg_server_path", server_dir);

    /* boot the ftps perl script */
    mk_filename (tmp, "sbin/ftps.pl", NULL);
    if (perl_load_file (tmp))
      {
	g_printerr ("perl_load_file(%s) failed\n", tmp);
	send_reply (421,
		    "failed to boot strap perl engine, closing control connection\n"
		    "check var/log/ftps.err for more hints");
	quit (CL_ERROR);
      }
    else /* finally throw event */
    if (!perl_call_subf ("FTP::event", "si", "connect_pre", 220))
      quit (CL_ERROR);
  }

  while (close_server < 2)
    {
      handle_signals ();
      if (close_server)
	{
	  printf ("UUD %i\n", close_reason);
	  fgets (str, sizeof (str), stdin);
	}
      switch (read_ftp_command (str, sizeof (str), p, 31, &n))
	{
	case -1:
	  if (errno)
	    {
	      if (errno == EINTR)
		handle_signals ();
	      else
		{
		  errlog ("read_ftp_command(): %s (%i)", strerror (errno),
			  errno);
		  quit (CL_SOCKERR);
		}
	    }
	  break;

	case 1:
	  break;

	default:
        if (close_server) {
	      close_server++;
          switch (close_reason) {
		case CL_CLOSE:
              send_reply(421, "Server closing, closing control connection");
		  break;
		case CL_SHUTDOWN:
              send_reply(421, "Server shutdown, closing control connection");
		  break;
		case CL_TIMEOUT:
              send_reply(421, "Timeout, closing control connection");
		  break;
		case CL_KICK:
              send_reply(421,
			      "You have been kicked, closing control connection");
		  break;
		}
        } else {
          if (dispatch_command(str, sizeof(str), p, n) == -1) {
		  if (errno == EINTR)
              handle_signals();
		  else
              quit(CL_SOCKERR);
		}
	    }
	}			/* switch */
    }

#ifdef TLS
  tls_cleanup();
#endif

  return close_reason;
}

/* EOF */
