/*   login.c
 *
 *   This file is part of FTP4ALL
 *
 *   Copyright (C) 1996-1999 by Crescent
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <sys/types.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include "externs.h"
#include "common.h"
#include "config.h"
#include "ftps.h"
#include "log.h"
#include "perm.h"
#include "rfc931.h"
#include "utils.h"
#include "site_vfs.h"
#include "userdb.h"
#include "perl_layer.h"
#include "strlcpy.h"
#include "strlcat.h"

#ifdef TLS
#include "tlsutil.h"
#endif


/* Gets the group list from ftpd */
void get_group_list(void)
{
    struct group_s *g;
    int             i;

    printf("GRP_LSR\n");
    if (!scanf("%i", &group_count)) return;
    fgetc(stdin);
    if (!group_count) return;
    if (group_count > group_size) 
    {
	group_size = group_count;
	group = (struct group_s *) 
	    g_realloc(group, group_size * sizeof(struct group_s));
	if (!group) 
	{
	    errlog("not enough memory for group list");
	    group_size = group_count = 0;
	    printf("-\n");
	    return;
	}
    }
    printf("+\n");
    g = group;

    for (i=0; i<group_count; i++)
    {
	if (scanf("%s "SI32,g->name,&g->gid) != 2)
	{
	    errlog("error reading group list");
	    group_count = i;
	    return;
	}
	userdb_register_gid (g->gid, g->name);
	fgetc (stdin);
	g++;
    }
}


/* Gets the user list from the ftpd */
void get_user_list(void)
{ 
    int            i;
    struct user_s* u;

    printf("USR_LSR\n");
    if (!scanf("%i",&user_count)) return;
    fgetc(stdin);
    if (!user_count) return;
    if (user_count > user_size)
    { 
	user_size = user_count;
	user = (struct user_s*)g_realloc(user,user_size*sizeof(struct user_s));
	if (!user)
	{ 
	    errlog("not enough memory for user list");
    	    user_count = user_size = 0;
    	    printf("-\n");
    	    return;
	}
    }
    printf("+\n");
    u = user;
  
    for (i=0; i<user_count; i++)
    {
	if (scanf("%s "SI32,u->name,&u->uid) != 2)
	{
	    errlog("error reading user list");
	    user_count = i;
	    return;
        }

	userdb_register_uid (u->uid, u->name);
	fgetc(stdin);
	u++;
    }
}


void get_dupecheck(void)
{ 
    char str[512];

    printf("DUP\n");
    readline(str,sizeof(str),stdin);
    if (*str == '+') dupechk = strdup(str+1);
}


int get_message(char* cmd, char** message, int* msg_len, int* msg_size)
{ 
    int n;
    int mlen = 0;
    int msize = 0;

    if (!msg_len) msg_len = &mlen;
    if (!msg_size) msg_size = &msize;
    printf("%s\n",cmd);
    scanf("%i",msg_len);
    fgetc(stdin);
    if (!*msg_len) return 1;
    if (*msg_len >= *msg_size)
    { 
	*msg_size = *msg_len+1;
	*message = g_realloc(*message,*msg_size+2);
    }
    if (!*message)
    { 
	printf("-\n");
	errlog("out of memory (get_message)");
	return 0;
    }
    printf("+\n");
    n = 0;
    while (n < *msg_len)
    { 
	if (!fgets(*message+n,*msg_size-n+2,stdin))
	{ 
	    errlog("fgets error (get_message)");
    	    return 0;
	}
	n += strlen(*message+n);
    }
    *(*message+*msg_len) = 0;
    return 1;
}


/* Handles the USER command */
int cmd_user(char** p, int n)
{ 
    int  anon_login;
    char name[sizeof(usr.name)+2];
    char str[1024];
    char wd[256];
    int fastlogin = 0;
    struct user_s* u;
    char loginstr[16];
    int maxchars;

#ifdef TLS
    if (tls_force_prot && !tls_on_ctrl)
    {
	return send_reply(501, "Site requires use of AUTH TLS.");
    }
#endif

    /* logged in already */
    if (*usr.name)
    {
	return send_reply(530,"Cannot change user id, log out first.");
    }
    if (n != 1) return send_reply(501,"User name required.");
  
    /* use only sizeof(usr.name) chars for login and drop the rest */
    /* attention: ! and $ need to be added to maxchars in this case */
    maxchars = sizeof(usr.name);
    if ((*p[0] == '!') || (*p[0] == '$')) 
    {
	maxchars++;
    }
  
    if ((*(p[0]+1) == '!') || (*(p[0]+1) == '$')) 
    {
	maxchars++;
    }
    strlcpy(name, *p, maxchars);
    *usr.name = '\0';

    /* get anonymous and ident flags */
    printf("UAN %s\n", name);
    fgets(str, sizeof(str), stdin);
    anon_login = (str[0] == '+');
    anonymous = (str[1] == 'a');
    if (anonymous) 
    {
	if (!anon_login) 
	{
	    log_action(0, LG_SECURITY, "LOGIN",
		"by %s FAILED: Guest login disabled", name);
	    return send_reply(530, "Guest login disabled.");
	}
	strlcpy(str, "Guest login initiated, send E-Mail address as password.",
	    sizeof(str));
#ifdef TLS
    } 
    else 
    {
	char *tls_user = tls_userid_from_client_cert();
	if (tls_user && !strcmp(tls_user, name))
	{
	    tls_pass_passthrough = 1;
	}
	//  else if (tls_is_user_valid(name))
	//    tls_pass_passthrough = 1;
	else
	{
	    tls_pass_passthrough = 0;
	}
	if (tls_pass_passthrough) 
	{
	    log_action(0, LG_SECURITY, "LOGIN", 
		"by %s SUCCEEDED: TLS/X509 authentication.", name);
#endif
    } 
    else 
    {      
      g_snprintf(str, sizeof(str), "Password required for %s.", name);
    }
#ifdef TLS
    }
#endif

    if (send_reply(331, str) == -1) return -1;

    /* get password */
#ifdef TLS
    if (tls_pass_passthrough)
    {
	n = -1;
	p[0] = strdup("PASS");
	if (!p[0]) exit(1);
    } 
    else
#endif
    switch (read_ftp_command(str, sizeof(str), p, 4, &n)) 
    {
	case -1:
	    return -1;
	case 0:
	    break;
	default:
	    return 0;
    }
    
    if (strcasecmp(*p, "PASS"))
    {
	return send_reply(503, "Please login with USER and PASS.");
    }

    if (n > 2)
    {
	return send_reply(501, "Syntax: PASS <password>");
    }

    if (anonymous)
    {
	if (n == 2)
	{
    	    anonpass = strdup(p[1]);
	}
    }
	
#ifdef TLS
    if (n == -1)
    {
	printf("ULP %s %s <none> 1\n", usr.ident, name);
    }
#endif
    if ((n == 1) && (!anonymous))
    {
	printf("ULP %s %s\n", usr.ident, name);
    }
    else
    {
	printf("ULP %s %s %s\n", usr.ident, name,
	    (anonymous) ? ((!anonpass) ? "<none>" : anonpass) : p[1]);
    }

    if (!fgets(str, sizeof(str), stdin)) 
    {	
	/* probably login kick */
	quit(CL_NORMAL);
	return send_reply(421, "Server has closed connection");
    }

    if (*str == '-') 
    {
	strlcpy(name, str + 3, sizeof(name));
	retries++;
	if (login_retries && (retries >= login_retries)) 
	{
    	    quit(CL_NORMAL);
    	    log_action(0, LG_SECURITY, "LOGIN",
		"by %s FAILED: Too many login failures", name);
	    return send_reply(421, "Too many login failures, connection closed.");
	}
	switch (tolower(str[1])) 
	{
	    case '-':
		return send_reply(530, "Pipe error, login failed.");
	    case 'a':
		return send_reply(530, "You are already logged in maximum times.");
	    case 'c':
		quit(CL_NORMAL);
		return send_reply(421, "Server is temporary closed.");
	    case 'd':
		quit(CL_NORMAL);
		return send_reply(421, "Your account has been disabled.");
	    case 'g':
		quit(CL_NORMAL);
		return send_reply(421,
		    "Your group is already logged in maximum times.");
	    case 'i':
		return send_reply(530,
		    "You are not allowed to log in from this "
		    "Ident@IP.");
	    case 'n':
		return send_reply(530, "Anonymous login is disabled.");
	    case 'p':
		return send_reply(530, "Login incorrect.");
	    case 's':
		quit(CL_NORMAL);
		return send_reply(421, "Shutdown in progress, no logins allowed.");
	    case 't':
		quit(CL_NORMAL);
		sleep(5);
		return send_reply(421,
		    "Too many users are logged in at the moment. Try again later.\r\n");
	    case 'u':
		quit(CL_NORMAL);
		return send_reply(421, "No more connections from your IP/domain.");
	    default:
		return send_message(530, 1, "Login failed, reason unknown (%s).",
                            str);
	}
    }

    sscanf(str + 1,
	"%s " SI32 " " SI32 " " SI64 " " SI64 " " SITIME " %i " SI32 " " SI64
	" " SI32 " " SI32 " " SI32 " " SI32 " " SI32 " " SI32 " " SI32 " "
	SI32 " " SI32 " " SI32 " " SI32 " %i %i", usr.name, &usr.uid,
	&usr.gid, &usr.downloaded, &usr.uploaded, &usr.last_login, &timeout,
	&usr.limit, &usr.credit, usr.gids, usr.gids + 1, usr.gids + 2,
	usr.gids + 3, usr.gids + 4, usr.gids + 5, usr.gids + 6, usr.gids + 7,
	&usr.class, &dl_speed, &ul_speed, &fastlogin, &user_locked);

    if (!fgets(str, sizeof(str), stdin)) 
    { 
	/* u->info */
	quit(CL_NORMAL);
	return send_reply(421, "Server has closed connection");
    } 
    else if (*str == '-') 
    {
	return send_reply(530, "Pipe error, login failed.");
    } 
    else 
    {
	strlcpy(usr.info, str + 1, sizeof(usr.info));
	if (*(usr.info + strlen(usr.info) - 1) == '\n')
	{
	    *(usr.info + strlen(usr.info) - 1) = 0;
	}
    }

    /* home directory */
    if (!fgets(wd, sizeof(wd), stdin)) 
    {	
	/* u->home */
	quit(CL_NORMAL);
	return send_reply(421, "Server has closed connection");
    } 
    else if (*wd == '-')
    {
	return send_reply(530, "Pipe error, login failed.");
    }
    else 
    {
	strlcpy(usr.home, wd + 1, sizeof(usr.home));
	if (*(usr.home + strlen(usr.home) - 1) == '\n')
	{
	    *(usr.home + strlen(usr.home) - 1) = 0;
	}
    }

    errlog("arg: %s\n", str);
    /* init VFS layer */
    {
	gint ngids;
	for (ngids = 0; ngids < 8; ngids++)
	{
    	    if (usr.gids[ngids] == -1)
	    {	
		break;
	    }
	}
	userdb_setuser(usr.uid, usr.gid, ngids, usr.gids);
    }
    vfs_set_superuser(superuser ());
  
    if (usr.home[0] != '-') 
    {
	gint rc = user_locked ? vfs_chroot (usr.home) : 
	    vfs_setcwd (usr.home);
	if (rc)
	{
	    errlog("Login %s: error while setting homedir '%s': %s\n", 
		usr.name, usr.home, vfs_strerror (vfs_errno));
	}
	if (rc && user_locked)
	{
	    quit (CL_NORMAL);
	    return 0;
	}
    }

/*  if (!perl_call_subf ("FTP::event", "sii", "login_pre", 230, fastlogin))
    {
      quit (CL_NORMAL);
      return 0;
    }*/

    logged_in = 1;
/*   prepare_msg(&cdup_msg); */
/*   prepare_msg(&chdir_msg); */
    prepare_msg(&dele_msg);
    prepare_msg(&download_msg);
/*   prepare_msg(&goodbye_msg); */
/*   prepare_msg(&list_msg); */
    prepare_msg(&mkdir_msg);
    prepare_msg(&rmdir_msg);
/*   prepare_msg(&toomany_msg); */
    prepare_msg(&upload_msg);
/*   prepare_msg(&welcome_msg); */
    prepare_msg(&nuke_msg); 
    prepare_msg(&unnuke_msg); 
    get_group_list();
    get_user_list();
  /* get_dupecheck(); */
  /* get_shortcut(); */
    load_cdpath();
    load_check();
    load_dcheck();
    load_dirdcheck();
    load_delcheck();
    load_dshortcut();
    load_customcmd();
  
    {
	/* fixme */
	gchar str[256] = { 0, };
	set_enviroments(str, sizeof(str), NULL);
    }

    if (!perl_call_subf("FTP::event", "sii", "login_post", 230, fastlogin))
    {
	quit (CL_NORMAL);
	return 0;
    }
    snprintf(loginstr,sizeof(loginstr), "%d", time(NULL));
    perl_set_var_str ("FTP::var_logintime", loginstr);

    return 0;
}


/* Handles the PASS command */
int cmd_pass(char** p, int n)
{ 
    return send_reply(503,"Login with USER first.");
}


/* Handles the ACCT command */
int cmd_acct(char** p, int n)
{ 
    return send_reply(502,"ACCT command not implemented.");
}


/* Handles the QUIT command */
int cmd_quit(char** p, int n)
{
    if (logged_in) 
    {
	usr.downloaded += bytes_downloaded;
	usr.uploaded += bytes_uploaded;  
	perl_call_subf ("FTP::event", "si", "logout_post", 221);
    }
    quit (CL_NORMAL);
    return 0;
}

/* EOF */
