#!/usr/bin/perl -w

##############################################################################
#                                                                            #
#  DELFiLE ANNOUNCER (c) 2002 fejf                                           #
#                                                                            #
#  This program is free software; you can redistribute it and/or modify      #
#  it under the terms of the GNU General Public License as published by      #
#  the Free Software Foundation; either version 2 of the License, or         #
#  (at your option) any later version.                                       #
#                                                                            #
##############################################################################

my $standarddir   = $ENV{'F4A_STD'};

# ----------------------------------------------------------------------------
#  short name of your site for announces (ignored if set in registry)
# ----------------------------------------------------------------------------
my $sitename      = "XYZ";


# ----------------------------------------------------------------------------
#  pidfile of your eggdrop sitebot (ignored if set in registry)
# ----------------------------------------------------------------------------
my $botpidfile    = $standarddir."/../sitebot/pid.Jackie_B";


# ----------------------------------------------------------------------------
# text file for announce.tcl (ignored if set in registry)
# ----------------------------------------------------------------------------
my $announcefile  = $standarddir."/../sitebot/announce.txt";


# ----------------------------------------------------------------------------
# #  these directories are no releases and should be handled in a special way
# # ----------------------------------------------------------------------------
my $special_dirs = '(cd[0-9]|sample|disc[0-9])';


# ----------------------------------------------------------------------------
#  you don't have to change these settings usually
# ----------------------------------------------------------------------------
my $filledfile = ".filled";
my $statusfile = ".status";


##############################################################################
# end of configuration (don't edit below unless you know what you are doing  #
##############################################################################

use strict;

use Cwd;
use Fcntl;
use Time::Local;
use File::Basename;

use FindBin;
use lib "$FindBin::Bin/";
use primetools;

my(@dirinfo);
my(%registry, %dircfg);
my($parentdir, $fullpath, $announcefilename);
my($username, $groupname, $section, $tagline, $gf, $tf);

# ----------------------------------------------------------------------------
#   update_filled($filename)
#   return 1 on error else 0, remove one file from the filled file
# ----------------------------------------------------------------------------
sub update_filled($) {
	my $filename   = shift;
	my $needrewrite = 0;
	my @oldfile   = ();
	my @newfile   = ();
	
	sysopen(FILLEDFILE, $filledfile, O_RDWR|O_CREAT) or return 1;
	# LOCK_EX = 2
	flock(FILLEDFILE, 2) or return 1;
	while (<FILLEDFILE>) { 
		push(@oldfile, $_);
	}
	foreach my $line (@oldfile) {
		chomp($line);
		if ((split(/\s+/, $line, 7))[6] !~ /\Q$filename\E/i) {
			push(@newfile, $line);
		}
		else {
			$needrewrite=1;
		}
	}
	if($needrewrite == 0) {
		# LOCK_UN = 8
		flock(FILLEDFILE, 8);
		close(FILLEDFILE);
		return 0;
	}
	truncate (FILLEDFILE, 0) or return 1;
	# SEEK_SET = 0
	seek (FILLEDFILE, 0, 0) or return 1;
	foreach my $line (@newfile) { 
		print FILLEDFILE "$line\n"; 
	}
	# LOCK_UN = 8
	flock(FILLEDFILE, 8);
	close(FILLEDFILE);
	if (exists $dircfg{announce_delfile} and $dircfg{announce_delfile} eq "yes") {
		if(-e $statusfile) {
			sysopen(STATUSFILE, $statusfile, O_RDWR) or return 1;
			# LOCK_EX = 2
			flock(STATUSFILE, 2) or return 1;
			($gf, $tf)=split /\s+/, <STATUSFILE>;
			truncate (STATUSFILE, 0) or return 1;
			# SEEK_SET = 0
			seek (STATUSFILE, 0, 0) or return 1;
			print STATUSFILE ($gf-1)." $tf";
			# LOCK_UN = 8
			flock(STATUSFILE, 8);
			close STATUSFILE;
			if(($tf) and ($gf == $tf)) {
				print " Announcing ...\n";
				announce_delfile();
			}
		}
	}
# TODO an update for the status-dir would be fine...
	return 0;
}

# ----------------------------------------------------------------------------
#   announce_newdir() - what do you think is this doing ? 
# ----------------------------------------------------------------------------
sub announce_delfile {
   my $formatfile = $standarddir."msg/irc/delfile";
   my $formatstring = "";
   my @rules;

   open(FORMATFILE, "< $formatfile") or return 0;
   $formatstring = <FORMATFILE>;
   close(FORMATFILE);
   chomp($formatstring);

   (my $minutes, my $hours) = (localtime)[1..2];
   my $time = sprintf("%02d:%02d", $hours, $minutes);

	@rules = (
		[ '$sitename'		, "$sitename"		],
		[ '$section'		, "$section"		],
		[ '$file'		, "$announcefilename"	],
		[ '$username'		, "$username"		],
		[ '$groupname'		, "$groupname"		],
		[ '$parentdir'		, "$parentdir"		],
		[ '$time'		, "$time"		],
		[ '$tagline'		, "$tagline"		]
	);
	announce_line($announcefile, $botpidfile, substitute($formatstring,0,@rules));
}


###############################################################################
# main program                                                                #
###############################################################################

	my $cwd = getcwd() or die "unable to get current working directory\n";

# read the ftp4all username and groupname from environment variables
	if (exists $ENV{'F4A_USER'}) { 
		$username = $ENV{'F4A_USER'}
	}
	else { 
		$username = "unknown"
	}
	if (exists $ENV{'F4A_GROUP'}) {
		$groupname = $ENV{'F4A_GROUP'}
	}
	else {
		$groupname = "unknown"
	}
	if (exists $ENV{'F4A_SECTION'}) {
		$section = $ENV{'F4A_SECTION'}
	}
	else {
		$section = "default"
	}
	if (exists $ENV{'F4A_INFO'}) {
		$tagline = $ENV{'F4A_INFO'}
	}
	else { 
		$tagline = "No tagline"
	}
#read registry and dircfg to global hashes
	%registry = read_registry(); 

	if (exists $registry{botpidfile}) { 
		$botpidfile = $registry{botpidfile}; 
	}
	if (exists $registry{announcefile}) { 
	$announcefile = $registry{announcefile}; 
	}

	$announcefilename = basename($ARGV[0]);

	@dirinfo = split(/\//, $cwd);
	if (($#dirinfo >= 2) and ($dirinfo[-1] =~ /^$special_dirs$/i)) {
		$parentdir = $dirinfo[-2]."_(".$dirinfo[-1].")";
	}
	else {
		$parentdir = $dirinfo[-1];
	}

	if (exists $registry{shortsitename}) {
		$sitename = $registry{shortsitename};
	}
	if (exists $registry{basedir}) {
	$fullpath = $registry{basedir} . $ARGV[0];
	}
	else {
		print "ERROR: unable to find basedir in the FTP4ALL registry file.\n";
		exit 1;
	}

	%dircfg = read_dircfg(dirname($fullpath));

	update_filled($announcefilename);

	print " File was successfully removed.\n";
	exit 0;

# --- end of file ---
