#!/usr/bin/perl -w

##############################################################################
#                                                                            #
#  primeCHECK v2.0.6 - FORCE SFV (c) 2000 primemover                         #
#                                                                            #
# ---------------------------------------------------------------------------#
#                                                                            #
#  This program is free software; you can redistribute it and/or modify      #
#  it under the terms of the GNU General Public License as published by      #
#  the Free Software Foundation; either version 2 of the License, or         #
#  (at your option) any later version.                                       #
#                                                                            #
##############################################################################


# ----------------------------------------------------------------------------
#  these filetypes are checked against a sfv file (regular expression)
# ----------------------------------------------------------------------------
my $sfvtypes = '\.(rar|[rs0-9][0-9][0-9]|mp3)';


##############################################################################
# end of configuration (don't edit below unless you know what you are doing  #
##############################################################################

use strict;
use Cwd;
use File::Basename;
use Time::Local;

use FindBin;
use lib "$FindBin::Bin/";
use primetools;


###############################################################################
# main program                                                                #
###############################################################################

if ($#ARGV < 0) {
   print "Syntax: force_sfv.pl <filename>\n";
   exit 1;
}

my $cwd = getcwd . '/' . dirname($ARGV[0]);
my $filename = basename($ARGV[0]);

# read registry and directory config file
# %registry = read_registry();
my %dircfg = read_dircfg($cwd); 


if (exists $dircfg{lock_complete} and $dircfg{lock_complete} eq "yes") {
   if ($filename !~ /.+\.nfo/i and -e ".locked") {
      printcolor("!BWARNING: this release is complete and the directory locked!!0");
      exit 1;
   }
}

if (exists $dircfg{enforce_sfv} and $dircfg{enforce_sfv} eq "yes" and $filename =~ /$sfvtypes$/i) {
   opendir(CWD, $cwd);
   my @allfiles = readdir(CWD);
   closedir(CWD);
   my @sfvfiles = grep { /\.sfv$/i} @allfiles;
   foreach my $file (@allfiles) {
	if ($file =~ /^\Q$filename\E$/i)
	{
		printcolor("!BWARNING: file $filename already present in directory, skip !0");
		exit 1;
	}
   }
   foreach my $sfvfile (@sfvfiles) {   
      open(SFVFILE, "< $cwd/$sfvfile") or next;
      while (<SFVFILE>) {
	 my $sfvf = (split / /)[0];
	 if(lc $sfvf eq lc $filename) {
	    print "$filename was found in sfv. upload permitted.\n";
            close(SFVFILE);
	    exit 0;
	 }
      }
      close(SFVFILE);
   }
   printcolor("!BWARNING: filename was not found in sfv but sfv-check is enforced.!0");
   exit 1;
}

exit 0;

# --- end of file ---
