# $Id: perms.pm,v 1.2 2001/04/13 10:27:23 soulshock Exp $

package perms;
require Exporter;

@ISA         = qw(Exporter);
@EXPORT      = qw(mode_str_ls mode_str_fstat);

@VFS_DF = ("hidden", "???", "???", "???", "hidden_r", "???", "???", "???");
@VFS_DP = ("cd", "ls", "mkdir", "rmdir", "get", "put", "overwrite", "chmod");
@VFS_FP = ("???", "write", "read", "chmod", "???", "???", "???", "???");

@VFS_TYPE = (0, "file", "directory", "link");

sub mode_str_ls {
  my ($type, $mode) = @_;

  my $result = "";

  if ($type == 1) {		# file
    $result .= '-';

    for (my $i = 0; $i < 3; $i++) {
      my $p = ($mode >> ((2-$i)*8)) & 0xff;
      $result .= $p & 0x04 ? 'r' : '-';
      $result .= $p & 0x02 ? 'w' : '-';
      $result .= $p & 0x08 ? 'x' : '-';
    }
  } elsif ($type == 2) {			# dir
    $result .= 'd';

    for (my $i = 0; $i < 3; $i++) {
      my $p = ($mode >> ((2-$i)*8)) & 0xff;
      $result .= $p & 0x10 ? 'r' : '-';
      $result .= $p & 0x20 ? 'w' : '-';
      $result .= $p & 0x01 ? 'x' : '-';
    }
  } else {
    $result .= '?';
    for (my $i = 0; $i < 3; $i++) {
      $result .= '---';
    }
  }

  return $result;
}

# helper functions

sub _dflag2str {
  my $result = "";

  return "none" if ($_[0] == 0x00);

  for (my $i = 0; $i < 8; $i++) {
    $result .= ($VFS_DF[$i] . " ") if ($_[0] & (1 << $i));
  }

  return $result;
}

sub _fmode2str {
  my $result = "";

  return "all" if ($_[0] == 0x0e);
  return "none" if ($_[0] == 0x00);

  for (my $i = 0; $i < 8; $i++) {
    $result .= ($VFS_FP[$i] . " ") if ($_[0] & (1 << $i));
  }

  return $result;
}

sub _dmode2str {
  my $result = "";

  return "all" if ($_[0] == 0xff);
  return "none" if ($_[0] == 0x00);

  for (my $i = 0; $i < 8; $i++) {
    $result .= ($VFS_DP[$i] . " ") if ($_[0] & (1 << $i));
  }

  return $result;
}

sub mode_str_fstat {
  my ($type, $mode) = @_;

  my @result;

  if ($type == 1) {		# file
    @result = (_fmode2str (($mode >> 16) & 0xff),
	       _fmode2str (($mode >> 8) & 0xff),
	       _fmode2str (($mode >> 0) & 0xff));
  } elsif ($type == 2) {	# dir
    @result = (_dmode2str (($mode >> 16) & 0xff),
	       _dmode2str (($mode >> 8) & 0xff),
	       _dmode2str (($mode >> 0) & 0xff),
	       _dflag2str (($mode >> 24) & 0xff));
  } else {			# unexpected
    @result = ();
  }

  return @result;
}
