# $Id: utils.pm,v 1.2.4.15 2002/07/16 19:15:59 pedro Exp $

package utils;
require Exporter;

@ISA         = qw(Exporter);
@EXPORT      = qw(
		  addtodirlog
		  announce_line
		  convertcolors
		  current_time_to_string 
		  filesize_to_string_short 
		  filesize_to_string_long 
		  get_formatfile
		  open_format
		  openftpd_chmod
	          openftpd_chown
		  printcolor
		  read_dircfg
		  read_registry 
		  strip_color
		  strdifftime 
		  strip_color 
		  stringtotime
		  substitute
		  trim
		  timetostring
		  $standarddir
		  $u $b $c
		 );

use strict;
use Cwd;
use Time::Local;
use File::Basename;
use Data::Dumper;
use Bit::Vector::Overload;
Bit::Vector->Configuration("input=decimal");
Bit::Vector->Configuration("operators=arithmetic");

my $DEBUG          = 0;

my $SHOWCOLORS     = 1;

# make sure this is exactly the size you used while compiling openftpd
my $MAXFILENAMELEN = 128;

# name of you permission files
my $PERMISSIONFILE = ".permissions";

# default uid used for new files
my $DEFAULT_UID    = 0;

# default gid used for new files
my $DEFAULT_GID    = 0;

my $standarddir = `cat $FTP::cfg_server_path/.openftpdrc`;
chop($standarddir);

# ----------------------------------------------------------------------------
#  General Substitutions for announces (this is done with every announce)
# ----------------------------------------------------------------------------
# chars for toggle bold and underline in irc output
my $u = pack("c", 31);
my $b = pack("c", 2);
my $c = pack("c", 3);
my @substrules = (
	[ '<b>'		, "$b"		],
	[ '</b>'	, "$b"		],
	[ '<u>'		, "$u"		],
	[ '</u>'	, "$u"		],
	[ '<br>'	, "\n"		],
	[ '<col>'	, "$c"		],
	[ '</col>'	, "$c"		],
	[ '<col0>'	, "$c".0	],
	[ '<col1>'	, "$c".1	],
	[ '<col2>'	, "$c".2	],
	[ '<col3>'	, "$c".3	],
	[ '<col4>'	, "$c".4	],
	[ '<col5>'	, "$c".5	],
	[ '<col6>'	, "$c".6	],
	[ '<col7>'	, "$c".7	],
	[ '<col8>'	, "$c".8	],
	[ '<col9>'	, "$c".9	],
	[ '<col10>'	, "$c".10	],
	[ '<col11>'	, "$c".11	],
	[ '<col12>'	, "$c".12	],
	[ '<col13>'	, "$c".13	],
	[ '<col14>'	, "$c".14	],
	[ '<col15>'	, "$c".15	]
);


# ----------------------------------------------------------------------------
#  filesize_to_string_short($filesize) - 
# ----------------------------------------------------------------------------
sub filesize_to_string_short {
    my $filesize = Bit::Vector->new_Dec(64, $_[0]);
    my $sizestr = "";
    if ($filesize->to_Dec > 1024) {
	if ($filesize->to_Dec > 1024*1024*1024) {
    	    $sizestr = sprintf("%.1fGB", $filesize->to_Dec / 1024 / 1024 / 1024);
	} else {
    	    $sizestr = sprintf("%.0fMB", $filesize->to_Dec / 1024 / 1024);
	}
    } else {
	$sizestr = sprintf("%.0fKB", $filesize->to_Dec / 1024.0);
  }
  return $sizestr;
}


# ----------------------------------------------------------------------------
#  filesize_to_string_long($filesize) - 
# ----------------------------------------------------------------------------
sub filesize_to_string_long {
    my $filesize = Bit::Vector->new_Dec(64, $_[0]);
    my $sizestr = "";
    if ($filesize->to_Dec > 1024) {
	if ($filesize->to_Dec > 1024 * 1024) {
    	    $sizestr = sprintf("%.1f MB", $filesize->to_Dec / 1024.0 / 1024.0);
	} else {
    	    $sizestr = sprintf("%.1f kB", $filesize->to_Dec / 1024.0);
	}
    } else {
	$sizestr = sprintf("%.1f bytes", $filesize->to_Dec);
    }
    return $sizestr;
}


# ----------------------------------------------------------------------------
#  strdifftime($earlier, $later) - calc the diff between two times as string
# ----------------------------------------------------------------------------
sub strdifftime {
   my $time1 = shift;
   my $time2 = shift;
   my $diffstr = "";  
   my $diff = $time2 - $time1;

   my $seconds = $diff % 60; $diff = ($diff - $seconds) / 60;
   my $minutes = $diff % 60; $diff = ($diff - $minutes) / 60;
   my $hours   = $diff % 24; $diff = ($diff - $hours) / 24;
   my $days    = $diff % 7; 
   my $weeks   = ($diff - $days) / 7;

   if ($weeks > 0) { return sprintf("%02dw", $weeks); }
   if ($days  > 0) { return sprintf("%02dd", $days);  }
   if ($hours > 0) { return sprintf("%02dh", $hours); }
   if ($minutes > 0) { return sprintf("%02dm", $minutes); }
   return sprintf("%02ds", $seconds); 
}


# ----------------------------------------------------------------------------
#  strip_color($string) - strip color codes and return clean string
# ----------------------------------------------------------------------------
sub strip_color {
	my $string = $_[0];
	$string =~ s/\!\w//g;
	$string =~ s/\!\!/\!/g;
	return $string;
}



# ----------------------------------------------------------------------------
#   open_format($filename) - opens format file and returns it
# ----------------------------------------------------------------------------
sub open_format {
	my $formatfile = shift;
	my $formatstring;
        open(FORMATFILE, "< $formatfile") or return 0;
	$formatstring = <FORMATFILE>;
        close(FORMATFILE);
        return ($formatstring);
}


# ----------------------------------------------------------------------------
#   read_registry() - return a hash with the whole registry
# ----------------------------------------------------------------------------
sub read_registry {
	my @lines = ();
	my ($key, $value, %KEYS);
	open(REGISTRY, "< $FTP::cfg_etc_path/ftpd.reg") or return(250,"unable to open $FTP::cfg_etc_path/ftpd.reg");
	while(<REGISTRY>) {
		if ($_ !~ /^#.+/) {
			chomp($_);
			push(@lines, $_);
		}
	}
	close(REGISTRY);
	for(@lines) {
		($key, $value) = split(/[ \t]/, $_, 2);
		if($value) {
			$value =~ s/^\t*//;
			$value =~ s/^\s*(.*?)\s*$/$1/;
			$KEYS{$key} = $value;
		}
	}
	return %KEYS;
}


# ----------------------------------------------------------------------------
#   announce_line($line) - announce a line to the sitebot
# ----------------------------------------------------------------------------
sub announce_line {
	my $announcefile = shift;
	my $botpidfile = shift;
	my $pid;

	open(ANNOUNCEFILE, ">> $announcefile") or return 0;
	print ANNOUNCEFILE shift, "\n";
	if ($DEBUG) { print "DEBUG: wrote line to announcefile $announcefile\n"; }
	close(ANNOUNCEFILE);

	open(PIDFILE, "< $botpidfile") or return 0;
	$pid = <PIDFILE>;
	chomp($pid);
	close(PIDFILE);
	kill 'HUP', $pid;
	return 1;
}


sub current_time_to_string {
	my ($day_of_month, $month, $year) = (localtime)[3..5];
	my $timestring = sprintf("%04d-%02d-%02d",  $year+1900, $month+1,
	                         $day_of_month);
	return $timestring;
}


# ----------------------------------------------------------------------------
#   timetostring($epoch_seconds) - returns string
#   create a string in YYYY-MM-DD HH:MM:SS format from a given time in
#   epoch seconds or from the current time if not parameter is given
# ----------------------------------------------------------------------------
sub timetostring {
	my ($seconds, $minutes, $hours, $day_of_month, $month, $year, $timestring);
	if ($_[0]) {
		($seconds, $minutes, $hours, $day_of_month, $month, $year) =
		(localtime($_[0]))[0..5];
	} else {
		($seconds, $minutes, $hours, $day_of_month, $month, $year) =
		(localtime)[0..5];
	}
	$timestring = sprintf("%04d-%02d-%02d %02d:%02d:%02d", $year+1900, $month+1,
	$day_of_month, $hours, $minutes, $seconds);
	return $timestring;
}

# ----------------------------------------------------------------------------
#   stringtotime($string) - returns time in epoch seconds
#   convert a given string in YYYY-MM-DD HH:MM:SS format into epoch seconds
# ----------------------------------------------------------------------------
sub stringtotime {
	my $epoch_seconds = 0;
	if ($_[0] =~ /(\d{4})-(\d{2})-(\d{2}) (\d{2}):(\d{2}):(\d{2})/) {
		$epoch_seconds = timelocal($6, $5, $4, $3, $2 - 1, $1 - 1900);
	}
	return $epoch_seconds;
}


# ----------------------------------------------------------------------------
#   addtodirlog($sec, $relpath)
# ----------------------------------------------------------------------------
sub addtodirlog {
    my $sec = shift;
    my $relpath = shift;
    my $directorylog = $FTP::cfg_log_path.'/directory.log';
    # Add it to the directory.log so it shows up in `site new`
    if (-e $directorylog) {
	open(DIRLOG, ">> $directorylog") or return (250, "ERROR: unable to open directory logfile.");
    } else {
	open(DIRLOG, "> $directorylog") or return (250, "ERROR: unable to open directory logfile.");
    }
    printf DIRLOG "%s %-10s %-10s %-16s %s\n", timetostring(), $FTP::var_user, $FTP::var_group, $sec, $relpath;
    close(DIRLOG);
}

# ----------------------------------------------------------------------------
#   read_dircfg($section) - return a hash with a whole section configuration
# ----------------------------------------------------------------------------
sub read_dircfg {
	my $section = shift;
	my %KEYS = ();
	my ($key, $value, $sectioncfg, @lines);

	# read sections.cfg
	local *SECCFG;
	if (not open(SECCFG, "< $FTP::cfg_etc_path/sections.cfg")) { 
		print "ERROR: unable to open sections.cfg\n";
		return ();
	}

	while(<SECCFG>) {
		# ignore comments and empty lines
		if ($_ !~ /^#|^\s*\n/) {
			chomp($_);
			if ($_ =~ /^$section\s+(\S+)\s+\S+\s+\S+/i) {
				$sectioncfg = $1;
				last;
			}
		}
	}
	close(SECCFG);

	print "reading $section from $sectioncfg\n";

	# file with defaults ...
	local *DIRCFG;
	if (not open(DIRCFG, "< $FTP::cfg_etc_path/sections/default.cfg")) { 
		print "ERROR: unable to open default.cfg\n";
		return ();
	}

	while(<DIRCFG>) {
		# ignore comments and empty lines
		if ($_ !~ /^#|^\s*\n/) {
			chomp($_); push(@lines, $_);
		}
	}
	close(DIRCFG);

	for (@lines) {
		($key, $value) = split(/[ \t]/, $_, 2);
		if ($key and $value) {
			$value =~ s/^\t*//;
			$value =~ s/^\s*(.*?)\s*$/$1/;
			$KEYS{$key} = $value;
		}
	}
    

	# ... then load the config file for the specific section
	if (open(DIRCFG, "< $FTP::cfg_etc_path/sections/$sectioncfg")) {
		@lines = ();
		while(<DIRCFG>) {
			if ($_ !~ /^#|^\s*\n/) {
				chomp($_); push(@lines, $_);
			}
		}
		close(DIRCFG);
		for (@lines) {
			($key, $value) = split(/[ \t]/, $_, 2);
			if ($key and $value) {
				$value =~ s/^\t*//;
				$value =~ s/^\s*(.*?)\s*$/$1/;
				$KEYS{$key} = $value;
			}
		}
	} else {
		print "ERROR: unable to open $section\n";
		return ();
	}
	return %KEYS;
}


# ----------------------------------------------------------------------------
#  strip_color($string) - strip color codes and return clean string
# ----------------------------------------------------------------------------
sub strip_color {
	my $string = $_[0];
	$string =~ s/\!\w//g;
	$string =~ s/\!\!/\!/g;
	return $string;
}


sub printcolor {
   if (defined $FTP::var_color and $FTP::var_color eq "c") {
      print $_[0];
   } else {
      if(length($_[0]) > 0) {
         print strip_color($_[0]);
      }
   }
}


sub convertcolors {
    $_[0] =~ s/<b>/$b/g; $_[0] =~ s/<\/b>/$b/g;
    $_[0] =~ s/<u>/$u/g; $_[0] =~ s/<\/u>/$u/g;

    if($SHOWCOLORS eq 1) {
	$_[0] =~ s/!a/\0030/g;
	$_[0] =~ s/!b/\0034/g;
	$_[0] =~ s/!c/\0033/g;
	$_[0] =~ s/!d/\0037/g;
	$_[0] =~ s/!e/\0032/g;
	$_[0] =~ s/!f/\0036/g;
	$_[0] =~ s/!g/\00310/g;
	$_[0] =~ s/!h/\00315/g;
	$_[0] =~ s/!A/\00314/g;
	$_[0] =~ s/!B/\0034/g;   # is there no light red? we'll use darkred for now.
	$_[0] =~ s/!C/\0039/g;
	$_[0] =~ s/!D/\0038/g;
	$_[0] =~ s/!E/\00312/g;
	$_[0] =~ s/!F/\00313/g;
	$_[0] =~ s/!G/\00311/g;
	$_[0] =~ s/!H/\0031/g;
	$_[0] =~ s/!1/\003,0/g;
	$_[0] =~ s/!2/\003,4/g;
	$_[0] =~ s/!3/\003,3/g;
	$_[0] =~ s/!4/\003,7/g;
	$_[0] =~ s/!5/\003,12/g;
	$_[0] =~ s/!6/\003,6/g;
	$_[0] =~ s/!7/\003,10/g;
	$_[0] =~ s/!8/\003,1/g;
	$_[0] =~ s/!0/\003/g;
	$_[0] =~ s/!X//g;
    } else {
	$_[0] =~ s/!a//g;
	$_[0] =~ s/!b//g;
	$_[0] =~ s/!c//g;
	$_[0] =~ s/!d//g;
	$_[0] =~ s/!e//g;
	$_[0] =~ s/!f//g;
	$_[0] =~ s/!g//g;
	$_[0] =~ s/!h//g;
	$_[0] =~ s/!A//g;
	$_[0] =~ s/!B//g;
	$_[0] =~ s/!C//g;
	$_[0] =~ s/!D//g;
	$_[0] =~ s/!E//g;
	$_[0] =~ s/!F//g;
	$_[0] =~ s/!G//g;
	$_[0] =~ s/!H//g;
	$_[0] =~ s/!1//g;
	$_[0] =~ s/!2//g;
	$_[0] =~ s/!3//g;
	$_[0] =~ s/!4//g;
	$_[0] =~ s/!5//g;
	$_[0] =~ s/!6//g;
	$_[0] =~ s/!7//g;
	$_[0] =~ s/!8//g;
	$_[0] =~ s/!0//g;
	$_[0] =~ s/!X//g;
    }
    return $_[0];
}

# ----------------------------------------------------------------------------
#   substitute ($string, @rules)
#     1. substitute with general rules
#     2. substitute with rules given as parameter
# ----------------------------------------------------------------------------
sub substitute {
	my $string = shift;
	my $exact = shift;
	my @rules = @_;
	my $dummy;

	$dummy = substituteit($string, 0, @substrules);
	return substituteit($dummy, 0, @rules);
}

sub substituteit {
	my $string = shift;
	my $exact = shift;
	my $nrules = $#_;

	for (my $i = 0; $i <= $nrules; $i++) {
		my $rule = shift;
		if ($rule) {
			my $s1 = quotemeta(@$rule[0]);
			my $s2 = @$rule[1];
			if ($s1) {
				if ($exact) {
					$string =~ s/^$s1$/$s2/;
				} else {
					$string =~ s/$s1/$s2/ig;
				}
			}
		}
	}
	return $string;
}


############################################################################
#                                                                          #
# OpenFTPD - CHMOD and CHOWN perl commands                                 #
#                                                                          #
############################################################################

my $PE_CHDIR     = 0x01;
my $PE_LIST      = 0x02;
my $PE_MKDIR     = 0x04;
my $PE_RMDIR     = 0x08;
my $PE_READ      = 0x10;
my $PE_WRITE     = 0x20;
my $PE_OVERWRITE = 0x40;
my $PE_CHMOD     = 0x80;
my $PE_ALL       = 0x7F;
my $PE_INVISIBLE = 0x01;
my $FPE_READ     = 0x04;
my $FPE_WRITE    = 0x02;
my $FPE_CHMOD    = 0x08;
my $FP_ALL       = 0x0E;
my $DP_HIDDEN    = 0x01;


sub trim {
	my @strings = @_;
	for (@strings) {
		s/^\s+//;
		s/\s+$//;
		s/\0+$//;
	}
	return wantarray ? @strings : $strings[0];
}


# --------------------------------------------------------------------------
# load permission file to a hash with filename as key ("." for directory)
# --------------------------------------------------------------------------
sub load_permissions {
	my $permfile = shift;
	my $filename;
	my %p;
	my $tmp;
	my (@fileperm, @dirperm);

	if (not open (PERMFILE, "< $permfile")) {
		if ($DEBUG) { print "Unable to open $permfile.\n"; }
		return %p;			
	}
	read (PERMFILE, $tmp, 48) || return -1;
	@dirperm = unpack "iiiiia28", $tmp;
	# print "@dirperm\n";
	$p{'.'}{'uid'}  = $dirperm[0];
	$p{'.'}{'gid'}  = $dirperm[1];
	$p{'.'}{'perm'} = $dirperm[2];
	$p{'.'}{'dlc'}  = $dirperm[3];
	$p{'.'}{'dlct'} = $dirperm[4];

	while (read PERMFILE, $tmp, 16 + $MAXFILENAMELEN) {
		@fileperm = unpack "a${MAXFILENAMELEN}iiii", $tmp;	
		$filename = trim($fileperm[0]);
		$p{$filename}{'uid'}  = $fileperm[1];
		$p{$filename}{'gid'}  = $fileperm[2];
		$p{$filename}{'dlc'}  = $fileperm[3];
		$p{$filename}{'perm'} = $fileperm[4];
	}
	close PERMFILE;
	if ($DEBUG) { 
		print "DEBUG: Permissionfile $permfile loaded ...\n"; 
	}
	return %p;
}


# --------------------------------------------------------------------------
# save data from hash to permission file
# --------------------------------------------------------------------------
sub save_permissions {
	my $filename = shift;
	my $foo = shift;
	my %p = %$foo;
	my ($tmp, $key, $uid, $gid, $dlc, $perm, $dlct);
	
	# first we write to a temporary file
	open (PERMFILE, "> ${filename}.tmp") || return 1;

	# we handle the dir permissions now
	if (exists $p{'.'}{'uid'}) { $uid = $p{'.'}{'uid'}; }
	else { $uid = $DEFAULT_UID; }
	if (exists $p{'.'}{'gid'}) { $gid = $p{'.'}{'gid'}; }
	else { $gid = $DEFAULT_GID; }
	if (exists $p{'.'}{'perm'}) { $perm = $p{'.'}{'perm'}; }
	else { $perm = 0; }
	if (exists $p{'.'}{'dlc'}) { $dlc = $p{'.'}{'dlc'}; }
	else { $dlc = 0; }
	if (exists $p{'.'}{'dlct'}) { $dlct = $p{'.'}{'dlct'}; }
	else { $dlct = 0; }

	$tmp = pack "iiiiia28", $uid, $gid, $perm, $dlc, $dlct,	"\0" x 28;
	print PERMFILE $tmp;

	# all the files in the list
	foreach $key (sort keys %p) {
		if (exists $p{$key}{'uid'}) { $uid = $p{$key}{'uid'}; }
		else { $uid = $DEFAULT_UID; }
		if (exists $p{$key}{'gid'}) { $gid = $p{$key}{'gid'}; }
		else { $gid = $DEFAULT_GID; }
		if (exists $p{$key}{'perm'}) { $perm = $p{$key}{'perm'}; }
		else { $perm = 0; }
		if (exists $p{$key}{'dlc'}) { $dlc = $p{$key}{'dlc'}; }
		else { $dlc = 0; }

		next if $key eq '.';
		$tmp = pack "a${MAXFILENAMELEN}iiii", 
			$key, $uid, $gid, $dlc, $perm;
		print PERMFILE $tmp;
	}
	close (PERMFILE);

	# now we rename the temporary to the final file
	rename "${filename}.tmp", $filename;

	if ($DEBUG) {
		print "DEBUG: Permissionfile $filename saved\n";
		print Dumper(%p);
	}
	return 0;
}


# --------------------------------------------------------------------------
# change_permissions(permission hash, filename, command)
# example: change_permissions(\%p, "myfile.zip", "ugo=rw");
# --------------------------------------------------------------------------
sub change_permissions {
    my $foo = shift;
    my %p = %$foo;
    my $command = shift;
    my $filename = shift;
    my $who;
    my ($what, @what);
    my $tmp;

    if ($command !~ /([ugod*]+)([+-=])([agpoclmnrwxh*]+)/) {
    	return %p;
    }
    $who = $1;
    $what = $3;
    $command = $2;
	
    if ($DEBUG) { print "DEBUG: $filename: $who $command $what\n"; }
    # the directory
    if ($filename eq '.') {
	if ($what =~ /a/) { push @what, $PE_ALL; }
	    if ($what =~ /g/) { push @what, $PE_READ; }
	    if ($what =~ /p/) { push @what, $PE_WRITE; }
	    if ($what =~ /o/) { push @what, $PE_OVERWRITE; }
	    if ($what =~ /c/) { push @what, $PE_CHDIR; }
	    if ($what =~ /l/) { push @what, $PE_LIST; }
	    if ($what =~ /m/) { push @what, $PE_MKDIR; }
	    if ($what =~ /n/) { push @what, $PE_RMDIR; }
	    if ($what =~ /\*/) { push @what, $PE_ALL; }
	    if (($who eq 'd') and ($what =~ /h/)) { 
	    	push @what, $DP_HIDDEN; 
	    }
	} else {
	    # file
	    if ($what =~ /r/) { push @what, $FPE_READ;  }
	    if ($what =~ /w/) { push @what, $FPE_WRITE; }
	    if ($what =~ /x/) { push @what, $FPE_CHMOD; }
	}	

	# add or remove permissions
	my $perm = 0;
	if ($command eq '+' or $command eq '-') {
	    # user
	    if ($who =~ /u/) { foreach my $w (@what) { $perm |= $w << 16; } }
	    # group
	    if ($who =~ /g/) { foreach my $w (@what) { $perm |= $w << 8; } }
	    # others
	    if ($who =~ /o/) { foreach my $w (@what) { $perm |= $w; } }
	    # user, group and others
	    if ($who =~ /\*/) {	
		foreach my $w (@what) {	$perm |= $w << 16; }
		foreach my $w (@what) { $perm |= $w << 8;}
		foreach my $w (@what) {	$perm |= $w; }
	    }
	    # hidden directory flag
	    if ($who =~ /d/) {
		foreach my $w (@what) {	$perm |= $w << 24; }
	    }
	    if ($command eq '+') {
		$p{$filename}{'perm'} |= $perm;
	    }
	    elsif ($command eq '-') {
	    	$p{$filename}{'perm'} &= ~$perm;
	    }
	}
	# set permissions
	elsif ($command eq '=') {
	    # user
	    $perm = 0;
	    if (exists $p{$filename}{'perm'}) {
	    	$perm = $p{$filename}{'perm'};
	    }
	    if ($DEBUG) { print "DEBUG: $perm\n";}
		
	    if ($who =~ /u/) {
	    	foreach my $w (@what) { $tmp += $w; }
	    	$perm = ($perm & 0xFF00FFFF) | ($tmp << 16);
	    }
	    $tmp = 0;
	    # group
	    if ($who =~ /g/) {
	    	foreach my $w (@what) {	$tmp += $w; }
	    	$perm = ($perm & 0xFFFF00FF) | ($tmp << 8);
	    }
	    $tmp = 0;
	    # others
	    if ($who =~ /o/) {
	    	foreach my $w (@what) {	$tmp += $w; }
		$perm = ($perm & 0xFFFFFF00) | $tmp;
	    }
	    $tmp = 0;
	    # user, group and others
	    if ($who =~ /\*/) {
	    	foreach my $w (@what) { $tmp += $w; }
		$perm = ($perm & 0xFF00FFFF) | ($tmp << 16);
		$tmp = 0;

		foreach my $w (@what) {	$tmp += $w; }
		$perm = ($perm & 0xFFFF00FF) | ($tmp << 8);
		$tmp = 0;

		foreach my $w (@what) {	$tmp += $w; }
		$perm = ($perm & 0xFFFFFF00) | $tmp;
		$tmp = 0;
	    }
	    $tmp = 0;
	    # hidden directory flag
	    if ($who =~ /d/) {
		foreach my $w (@what) { $tmp += $w; }
		$perm = ($perm & 0x00FFFFFF) | ($tmp << 24);
		$tmp = 0;
	    }
	    $p{$filename}{'perm'} = $perm;
	}
	if ($DEBUG) { 
	    print "DEBUG: Permissions changed!\n";
	}
	return %p;
}

# ----------------------------------------------------------------------------
#   get_formatfile ($event) 
# ----------------------------------------------------------------------------
sub get_formatfile {
	my($event, $formatstring);

	$event = shift;
	open(FORMATFILE, "< $standarddir/msg/irc/$event") or return undef;
	$formatstring = <FORMATFILE>;
	close(FORMATFILE);
	chomp($formatstring);
	return $formatstring;
}


# --------------------------------------------------------------------------
# openftpd_chmod(command, filename)
# example: openftpd_chmod("ugo=rw", "/home/site/somefile.zip");
# --------------------------------------------------------------------------
sub openftpd_chmod {
    my $command = shift;
    my $path = shift;
    my $permfile;
    my $filename = ".";
    my %permissions;
	
    if ($DEBUG) { print "----------- CHMOD: $path -----------------\n"; }

    # directory
    if (-d $path) { $permfile = "$path/$PERMISSIONFILE"; }
    # file
    elsif (-f $path) { 
    	$permfile = dirname($path) . "/$PERMISSIONFILE"; 
	$filename = basename($path);
    } else { 
	if ($DEBUG) { die "File $path was not found!\n"; }
	return; 
    }
    %permissions = load_permissions($permfile);
    if ($DEBUG and %permissions and exists $permissions{$filename}{'perm'}) { 
    	print "DEBUG: old permissions: ", 
	$permissions{$filename}{'perm'}, "\n";
    }
    %permissions = change_permissions(\%permissions, $command, $filename);
    if ($DEBUG and %permissions and exists $permissions{$filename}{'perm'}) { 
	print "DEBUG: new permissions: ", $permissions{$filename}{'perm'}, "\n";
    }
    save_permissions($permfile, \%permissions);	
}


# --------------------------------------------------------------------------
# openftpd_chown(uid, gid, filename)
# example: openftpd_chmod(2, 4, "/home/site/somefile.zip");
# --------------------------------------------------------------------------
sub openftpd_chown {
    my $uid = shift;
    my $gid = shift;
    my $path = shift;
    my %permissions;
    my $filename = ".";
    my $permfile;

    if ($uid !~ /\d+/ or $gid !~ /\d+/) { return; }

    # directory
    if (-d $path) { $permfile = "$path/$PERMISSIONFILE"; }
    # file
    elsif (-f $path) { 
    	$permfile = dirname($path) . "/$PERMISSIONFILE"; 
	$filename = basename($path);
    } else { 
    	if ($DEBUG) { die "File $path was not found!\n"; }
	return; 
    }
    %permissions = load_permissions($permfile);

    $permissions{$filename}{'uid'} = $uid;
    $permissions{$filename}{'gid'} = $gid;

    save_permissions($permfile, \%permissions);	
}

# --- end of file ---
