# $Id: events.pl,v 1.4.4.9 2002/05/10 13:50:28 fejf Exp $
# default core events

#
# documentation for event parameters
#
# ...to be written...
# return ($rc, $msg)
# $rc != 0 --> ok
# $rc == 0 --> deny (only meaningfull if pre event)
#
# by setting $_[0] the reply code can be modified
#

use lib "$FindBin::Bin/../lib";
use utils;

my $_retrieve_msg = sub {
    my $msgid = shift;
    my $msgfile;

    if ( -f "$FTP::cfg_msg_path/grp/$FTP::var_group/$msgid") {
	$msgfile = "$FTP::cfg_msg_path/grp/$FTP::var_group/$msgid";
    }
    if ( -f "$FTP::cfg_msg_path/usr/$FTP::var_user/$msgid") {
	$msgfile = "$FTP::cfg_msg_path/usr/$FTP::var_user/$msgid";
    }
    if (not $msgfile) {
	$msgfile = "$FTP::cfg_msg_path/$msgid";
    }
    if ( -f $msgfile ) {
	my $msgtext = readfile_expand ("$msgfile");
	return $msgtext;
    } else {
	return "msg file for '$msgid' not found";
    }
};

my $_ev_connect_pre = sub {
    return (1, &$_retrieve_msg ("startup"));
};

#my $_ev_login_pre = sub {
#  return (1, &$_retrieve_msg ("welcome"));
#};

my $_ev_login_post = sub {
    my $fastlogin;
    
    shift;
    $fastlogin = shift;
    
    if ($fastlogin eq "1") {
	return (1, "FASTLOGIN Activated\n");
    }
    else {
	return (1, &$_retrieve_msg ("welcome"));
    }
};

my $_ev_logout_post = sub {
    sub logtime {
	my ($minutes, $hours, $timestring);
	if ($_[0]) {
		($minutes, $hours) = (localtime($_[0]))[1..2];
	} else {
		($minutes, $hours) = (localtime)[1..2];
	}
	$timestring = sprintf("%02d:%02d", $hours, $minutes);
	return $timestring;
    }
    my (@lines, $line, $newline, $counter, $last);
    (my $ul_session, my $dl_session) = map { FTP::get_variable ($_) } qw/p d/;
    $ul_session  = filesize_to_string_short($ul_session);
    $dl_session  = filesize_to_string_short($dl_session);
    my $logon    = logtime($FTP::var_logintime);
    my $logout	 = logtime(time);
    my $duration = strdifftime($FTP::var_logintime,time);
    my $maxlines = 10;
    $newline = sprintf " %-8.8s | %-8.8s | %-20.20s |%-5.5s %-5.5s|%7.7s| %6.6s %6.6s\n",
		$FTP::var_user,$FTP::var_group,$FTP::var_tagline,$logon,$logout,$duration,$ul_session,$dl_session;
    # Open old file, if there is any
    $counter=1;
    local *OLDFILE;
    if (open (OLDFILE, "< $FTP::cfg_var_path/log/laston.log")) {
	while (<OLDFILE>) {
	    chomp $_;
	    push (@lines, $_);
	    $counter++;
	    last if ($counter == ($maxlines-1));
	}
    close OLDFILE;
    }
    # Create new file
    local *NEWFILE;
    if (open (NEWFILE, "> $FTP::cfg_var_path/log/laston.new")) {
	printf NEWFILE $newline;
	foreach $line (@lines) {
	    printf NEWFILE $line."\n";
	}
	close NEWFILE;
    }
    if (-f "$FTP::cfg_var_path/log/laston.new") {
	rename("$FTP::cfg_var_path/log/laston.new","$FTP::cfg_var_path/log/laston.log");
    }
    return (1, &$_retrieve_msg ("goodbye"));
};

my $_ev_chdir_post = sub {
    my ($header, $body, $footer, $grpheader);
    my ($filename, $line, $msgtext, $basedir, $count);
    my ($totalsize, $totalspeed, $totalfiles);
    my (%racers, %group, %size, %speed, %files);
    my (%grpmembers, %grpsize, %grpspeed, %grpfiles);
    my %reg = read_registry();
  
    if (exists $reg{basedir}) {
	$basedir= $reg{basedir};
    }
    $filename="$basedir". &FTP::vfs_getcwd() ."/.filled";
    $header=&$_retrieve_msg ("race_top");
    $body=&$_retrieve_msg ("race_body");
    $footer=&$_retrieve_msg ("race_tail");
    $grpheader=&$_retrieve_msg ("race_grp_top");
  
    $msgtext="";
    if(-f $filename) {
	open (FILE,"<$filename") or die;
	$line=<FILE>;				# 1st line is dir itself
	while ($line=<FILE>) {
    	    $line=~ /^\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2} (\S+)\s+(\S+)\s+(\d+)\s+(\d+)/;
    	    if($racers{$1}==1) {
		$size{$1}+=$3;
		$speed{$1}=($speed{$1}+$4)/2;		# better use some weighted calculation here (but needs realy BIG vars ;)
		$files{$1}++;
		$totalsize+=$3;
		$totalspeed=($totalspeed+$4)/2;		# better use some weighted calculation here (but needs realy BIG vars ;)
		$totalfiles++;
    	    } else {
    		$racers{$1}=1;
		$group{$1}=$2;
		$size{$1}=$3;
		$speed{$1}=$4;
		$files{$1}=1;
		$totalsize+=$3;
		$totalfiles++;
		if($totalspeed==0) {
		    $totalspeed=$4;
		} else {
		    $totalspeed=($totalspeed+$4)/2;	# better use some weighted calculation here (but needs realy BIG vars ;)
		}
    	    }
	}
    } 
    close FILE;

    $count=1;
    foreach $line (sort {$size{$b} <=> $size{$a}} keys %racers) {
	$msgtext.=sprintf($body, $count++, $line, $group{$line}, ($size{$line}/1048576), $files{$line}, ($speed{$line}/1024), ($size{$line}/$totalsize*100));
    }
    if($msgtext) {
	$count--;
	$footer=sprintf($footer, $count, ($totalsize/1048576), $totalfiles, ($totalspeed/1024));
	$msgtext=$header.$msgtext.$footer;
	# now create group-stats
	foreach $line (keys %racers) {
	    if($grpmembers{$group{$line}}) {
    		$grpmembers{$group{$line}}++;
		$grpfiles{$group{$line}}+=$files{$line};
		$grpsize{$group{$line}}+=$size{$line};
		$grpspeed{$group{$line}}=($grpspeed{$group{$line}}+$speed{$line})/2;
    	    } else {
		$grpmembers{$group{$line}}=1;
		$grpfiles{$group{$line}}=$files{$line};
		$grpsize{$group{$line}}=$size{$line};
		$grpspeed{$group{$line}}=$speed{$line};
    	    }
	}
	$msgtext.=$grpheader;
	$count=1;
	foreach $line (sort {$grpsize{$b} <=> $grpsize{$a}} keys %grpmembers) {
    	    $msgtext.=sprintf($body, $count++, $line, $grpmembers{$line}, ($grpsize{$line}/1048576), $grpfiles{$line}, ($grpspeed{$line}/1024), ($grpsize{$line}/$totalsize*100));
	}
	$count--;
	$footer=sprintf($footer, $count, ($totalsize/1048576), $totalfiles, ($totalspeed/1024));
	$msgtext.=$footer;
    }
    $msgtext.=sprintf ("Changed directory successfully to '%s'.", &FTP::vfs_getcwd ());
    return (1, $msgtext);
};

# bind party
FTP::bind_event ("connect_pre", $_ev_connect_pre);
#FTP::bind_event ("login_pre", $_ev_login_pre);
FTP::bind_event ("login_post", $_ev_login_post);
FTP::bind_event ("logout_post", $_ev_logout_post);
FTP::bind_event ("chdir_post", $_ev_chdir_post);
# EOF
