# OpenFTPD SITE PRE script 
# (c) 2001 Pedro / team openftpd
# pedro@openftpd.org

# 1. Every Section where people should be able to pre into must have an 
#    entry in it's sectionconfig. 
#    e.g. /openftpd/etc/sections/mp3.cfg:
#         prepath 	/mp3/incoming
#
#    all mp3-pre's go to <siteroot>/mp3/incoming
#
# 2. If your site has only one section to pre into, add to ftpd.reg:
#    predefault 	<pathtopre>
#    e.g.:
#    predefault 	MP3
 
# In what sections are your groupdirs (default is GROUPS) ?
my $grouppresection 	= "GROUPS";

# What filetypes will be counted for completeannounce?
my $prefiles		= '\.(?:rar|[rs0-9][0-9][0-9]|mp3|avi|mpg|m2v|zip)';

# Create complete dir/file 
##############################################################################
use Cwd;
use Cwd 'abs_path';
use File::Find;
use FindBin;
use lib "$FindBin::Bin/../lib";
use utils;
use mp3info;
use imdb;

my $pre_top      = &open_format($FTP::cfg_msg_path.'/pre_top');
my $pre_tail     = &open_format($FTP::cfg_msg_path.'/pre_tail');
my $prestatsfile = $FTP::cfg_var_path.'/prestats'; 
our (%reg, %dircfg);

# ----------------------------------------------------------------------------
#   announce_pre($botpidfile, $announcefile, $sitename, $directory, 
#		 $presection, $numfiles, $releasesize)
# ----------------------------------------------------------------------------
sub announce_pre {
	my $botpidfile = shift;
	my $announcefile = shift;
	my $sitename = shift;
	my $directory = shift;
	my $presection = shift;
	my $numfiles = shift;
	my $releasesize = shift;
	my $releasedir = shift;
	my $formatstring = "";
	my $formatfile;
	my $u = pack("c", 31);
	my $b = pack("c",2);
	my %mp3info;
	my @rules;
	my ($url, %imdb);

	# Check if presection is MP3 for id3tag announce
	if ($presection =~ /MP3/i) {
		find sub {
			if((not defined %mp3info) and ($File::Find::name =~ /\.mp3$/i)) {
				%mp3info = mp3_info($File::Find::name); 
			}
		}, $releasedir;
	}

	# Open format file
	if (defined %mp3info) {
		$formatstring = get_formatfile('pre_mp3');
	}
	else {
		$formatstring = get_formatfile('pre');
	}

	# Announce the pre 
	@rules = (
		[ '$sitename'		, "$sitename"		 ],
		[ '$directory'		, "$directory"		 ],
		[ '$username'		, "$FTP::var_user"	 ],
		[ '$groupname'		, "$FTP::var_group"	 ],
		[ '$section'		, "$presection"		 ],
		[ '$numfiles'		, "$numfiles"		 ],
		[ '$releasesize'	, "$releasesize"	 ],
		[ '$genre' 		, "$mp3info{'GENRE'}"	 ],
		[ '$year'		, "$mp3info{'YEAR'}"	 ],
		[ '$bitrate'		, "$mp3info{'BITRATE'}"	 ],
		[ '$frequency'		, "$mp3info{'FREQUENCY'}"]
	);
	if ($announcefile and $botpidfile and $formatstring) {
		announce_line($announcefile, $botpidfile, substitute($formatstring,0,@rules));
		if (($presection !~ /(MP3|APP|GAME|CONSOL)/i) and (exists $dircfg{announce_imdb}) and ($dircfg{announce_imdb} eq "yes")) {
			find sub {
				if((not defined $url) and ($File::Find::name =~ /\.nfo$/i)) {
					$url = get_imdb_url("$File::Find::name");
				}
			}, $releasedir;
			if ($url =~ 'imdb.com') {
				%imdb = parse_imdb($url);
				$formatstring = get_formatfile('imdb');
				if (!($imdb{plot})) {$imdb{plot}="n/a";}
				if (!($imdb{genre})) {$imdb{genre}="n/a";}
				if (!($imdb{tagline})) {$imdb{tagline}="n/a";}
				if (!($imdb{rating})) {$imdb{rating}="?";}
				if (!($imdb{votes})) {$imdb{votes}="no";}
				@rules = (
					[ '$sitename'	, "$sitename"		],
					[ '$directory'	, "$directory"		],
					[ '$parentdir'	, "$presection"		],
					[ '$username'	, "$FTP::var_user"	],
					[ '$groupname'	, "$FTP::var_group"	],
					[ '$numfiles'	, "$numfiles"		 ],
					[ '$title'	, "$imdb{title}"	],
					[ '$rating'	, "$imdb{rating}"	],
					[ '$votes'	, "$imdb{votes}"	],
					[ '$plot'	, "$imdb{plot}"		],
					[ '$genre'	, "$imdb{genre}"	],
					[ '$tagline'	, "$imdb{tagline}"	],
					[ '$url'	, "$url"		]
				);
				announce_line($announcefile, $botpidfile, substitute($formatstring, 0, @rules));
			}
		}
	}
}


###############################################################################

my $_site_pre = sub {
    my ($reg, $botpidfile, $announcefile, $basedir, $sitename, $dircfg);
    my ($releasesize, $matched, $predefault, $directory, $presection);
    my $releasesize = 0;
    my $numfiles = 0;
    my %mp3info = ();
    my $arg = shift;
    %reg = read_registry();

    $predefault = $reg{predefault} if exists $reg{predefault};

    # Check for illegal characters
    if ( $arg =~ /[;&\$\/\`](..)/) { #`
    	return (250, "ERROR: illegal meta characters in the arguments!");
    }

    ($directory, $presection) = split ' ', $arg;

    if ((not defined $directory) and (defined $predefault)){
        return (200, "!HSyntax: !cSITE PRE !C<!cdirectory!C>!0",
	             "!HPre <directory>!0");
    }
    elsif (((not defined $presection) or (not defined $directory)) and not defined $predefault) {
        return (200, "!HSyntax: !cSITE PRE !C<!cdirectory!C> !C<!csection!C>!0",
	             "!HPre <directory> into <section>!0");
    } 

    $presection    = $predefault             if defined $predefault;
    $botpidfile    = $reg{botpidfile}        if exists $reg{botpidfile};
    $announcefile  = $reg{announcefile}      if exists $reg{announcefile};
    $sitename      = uc($reg{shortsitename}) if exists $reg{shortsitename};

    %dircfg 	   = read_dircfg($presection);
    my $sitecwd    = &FTP::vfs_getcwd();
    my $section	   = $FTP::var_section;
    my $basedir    = $reg{basedir};
    my $group 	   = $FTP::var_group;
    my $realpredir = $dircfg{prepath};
    my $releasedir = $basedir.$dircfg{prepath}.'/'.$directory;

    # Check if destination dir/link exists
    if ( !-d $basedir.$dircfg{prepath}) {
	return (250, "Can't find destination directory ($directory) to PRE!");
    }

    # Check if source dir exists
    if( !-d $basedir.'/'.$sitecwd.'/'.$directory) {
	return (250, "Can't find directory ($directory) to PRE!");
    }

    # Check if groupdir is in presection (disabled if siteop or superuser wants to pre)
    # TODO: Check groups from user
    # if ((($sitecwd !~ /$group$/i) or ($section !~ /$grouppresection/i)) and (($FTP::var_superuser ne 1) or ($FTP::var_siteop ne 1))) {
    if ((($section !~ /$grouppresection/i)) and (($FTP::var_superuser ne 1) or ($FTP::var_siteop ne 1))) {
	return (250,"You are not in your group directory!");
    }

    # Check if destination section exists
    if (not defined %dircfg) {
	return (250, "Couldnt find section ($section)");
    }

    # Check if sectionconfig has "prepath" set
    if (not defined $dircfg{'prepath'}) {
	return (250, "Section ($presection) is not allowed to pre into");
    }

    # Check if directory already exists in destination dir
    if (-d $releasedir) {
	return (250, "You Prerelease already exists in destination, be faster next time!");
    }
    
    # first lets get some release info tho, #files, size.
    find sub { $numfiles++ if ($_ =~ /.+$prefiles/i)}, $basedir.$sitecwd.'/'.$directory;
    find sub { $releasesize += -s }, $basedir.$sitecwd.'/'.$directory;
    $releasesize = sprintf("%4.1f",($releasesize/1048576));

    # All good, go ahead and make it happen!
    system ('mv', $basedir.$sitecwd.'/'.$directory, $basedir.$dircfg{prepath});
    if ( -e $releasedir) {
	utime(time, time, $releasedir);
    } else {
	return (250, "Pre wasn't successful");
    }

    #add to dirlog
    addtodirlog (uc($presection), $realpredir.'/'.$directory);

    # Announce the PRE
    if (exists $reg{'botpidfile'} and exists $reg{'announcefile'}) {
	announce_pre($reg{'botpidfile'},
	             $reg{'announcefile'},
		     exists $reg{'shortsitename'} ? uc($reg{'shortsitename'}) : "???",
	    	     $directory,
	             uc($presection),
		     $numfiles,
	    	     $releasesize,
		     $releasedir);
    }
	# print pre to stats-file
	open (STATFILE, ">>$prestatsfile");
	printf STATFILE "$presection	$FTP::var_group	$numfiles	$releasesize	$directory\n";
	close STATFILE;
    # output Pre was successful
    return (200, $pre_top."\n\nFound section and directory! Now pre'ing...\nMoving $sitecwd/$directory\n-> $realpredir/$directory\n".$pre_tail);
};

&FTP::bind_site_cmd ("pre", $_site_pre);

# --- end of file ---
