##############################################################################
#                                                                            #
#  primeSTATS v3.0.1 - (c) 2001 primemover                                   #
#                                                                            #
#  frontend for OpenFTPD statfiles                                           #
# ---------------------------------------------------------------------------#
#                                                                            #
#  This program is free software; you can redistribute it and/or modify      #
#  it under the terms of the GNU General Public License as published by      #
#  the Free Software Foundation; either version 2 of the License, or         #
#  (at your option) any later version.                                       #
#                                                                            #
##############################################################################

my $ubody1  = "!H%2d!G|!H%-9.9s!G|!H%-9.9s!G|!H%-34.34s!G|!H%6.0f!G|!H%8.0f!G|!H%5.0f!0\n";
my $ubody2  = "!h%2d!G|!h%-9.9s!G|!h%-9.9s!G|!h%-34.34s!G|!h%6.0f!G|!h%8.0f!G|!h%5.0f!0\n";

my $gbody1  = "!H%2d!G|!H%-9.9s!G|!H%7d!G|!H%-36.36s!G|!H%6.0f!G|!H%8.0f!G|!H%5.0f!0\n";
my $gbody2  = "!h%2d!G|!h%-9.9s!G|!h%7d!G|!h%-36.36s!G|!h%6.0f!G|!h%8.0f!G|!h%5.0f!0\n";

my $show_global = 1;  # show always global stata instead if default section

#userstat header
my $uheader = &open_format($FTP::cfg_msg_path."/ustat_top");

#groupstat header
my $gheader = &open_format($FTP::cfg_msg_path."/gstat_top");

#userstat tail
my $ufooter = &open_format($FTP::cfg_msg_path."/ustat_tail");

#groupstat tail
my $gfooter = &open_format($FTP::cfg_msg_path."/gstat_tail");


##############################################################################

use Cwd;
use File::Basename;
use lib "$FindBin::Bin/../lib";
use utils;
use Data::Dumper;

sub load_stats {
	my %stats = ();
	my $section = shift;
	my $statfile = "$FTP::cfg_var_path/stats/" . lc($section) . ".txt";
	open(STATFILE, "< $statfile") or return 0;
	while (<STATFILE>) {
		my @s = split(/\t/, $_);
		if ($#s != 33) {
			die "ERROR: unknown stat file format.";
		}
		my $username = $s[0];
		$stats{$username}{userinfo}    = $s[1];
		$stats{$username}{groupname}   = $s[2];
		$stats{$username}{groupinfo}   = $s[3];

		$stats{$username}{download}{day}{files}   = $s[4];
		$stats{$username}{download}{day}{bytes}   = $s[5];
		$stats{$username}{download}{day}{seconds} = $s[6];

		$stats{$username}{upload}{day}{files}   = $s[7];
		$stats{$username}{upload}{day}{bytes}   = $s[8];
		$stats{$username}{upload}{day}{seconds} = $s[9];

		$stats{$username}{download}{week}{files}   = $s[10];
		$stats{$username}{download}{week}{bytes}   = $s[11];
		$stats{$username}{download}{week}{seconds} = $s[12];

		$stats{$username}{upload}{week}{files}   = $s[13];
		$stats{$username}{upload}{week}{bytes}   = $s[14];
		$stats{$username}{upload}{week}{seconds} = $s[15];

		$stats{$username}{download}{month}{files}   = $s[16];
		$stats{$username}{download}{month}{bytes}   = $s[17];
		$stats{$username}{download}{month}{seconds} = $s[18];

		$stats{$username}{upload}{month}{files}   = $s[19];
		$stats{$username}{upload}{month}{bytes}   = $s[20];
		$stats{$username}{upload}{month}{seconds} = $s[21];

		$stats{$username}{download}{year}{files}   = $s[22];
		$stats{$username}{download}{year}{bytes}   = $s[23];
		$stats{$username}{download}{year}{seconds} = $s[24];

		$stats{$username}{upload}{year}{files}   = $s[25];
		$stats{$username}{upload}{year}{bytes}   = $s[26];
		$stats{$username}{upload}{year}{seconds} = $s[27];

		$stats{$username}{download}{total}{files}   = $s[28];
		$stats{$username}{download}{total}{bytes}   = $s[29];
		$stats{$username}{download}{total}{seconds} = $s[30];

		$stats{$username}{upload}{total}{files}   = $s[31];
		$stats{$username}{upload}{total}{bytes}   = $s[32];
		$stats{$username}{upload}{total}{seconds} = $s[33];
	}
	close(STATFILE);
	return %stats;
}


sub print_user_stats {
	my $s = shift;
	my %stats = %$s;
	my $d = shift;      #direction
	my $p = shift;      #time period
	my $lines = shift;
	my $rank = 0;
	my $bodydata;
	
	foreach my $u (sort {$stats{$b}{$d}{$p}{'bytes'} <=> 
	                     $stats{$a}{$d}{$p}{'bytes'}} 
	               keys %stats) 
	{   
		my $mbsize = $stats{$u}{$d}{$p}{'bytes'} / 0x100000;
		my $speed  = $stats{$u}{$d}{$p}{'seconds'} > 0 ? 
		          $stats{$u}{$d}{$p}{'bytes'} / 
			  ($stats{$u}{$d}{$p}{'seconds'} * 1024) : 0;
		my $files  = $stats{$u}{$d}{$p}{'files'};
		my $group  = $stats{$u}{'groupname'};
		my $info   = $stats{$u}{'userinfo'};
		$rank++;
		
		if (($rank > $lines) or ($mbsize == 0)) { last; }

		my $fmt = $rank % 2 ? $ubody1 : $ubody2;
		$bodydata .= sprintf($fmt, $rank, $u . '.' x 9, $group  . '.' x 9, 
	                             $info  . '.' x 36, $files, $mbsize, $speed);
	}
	return $bodydata;
}


sub print_group_stats {
	my $s = shift;
	my %stats = %$s;
	my $d = shift;
	my $p = shift;
	my $lines = shift;
	my $rank = 0;
	my %gstats = ();
	my $bodydata;

	# sum groupstats
	foreach my $u (keys %stats) {
		my $g = $stats{$u}{'groupname'};
		if (exists $gstats{$g}) {
			$gstats{$g}{$d}{$p}{'files'} += 
			$stats{$u}{$d}{$p}{'files'};
			
			$gstats{$g}{$d}{$p}{'bytes'} += 
			$stats{$u}{$d}{$p}{'bytes'};
			
			$gstats{$g}{$d}{$p}{'seconds'} += 
			$stats{$u}{$d}{$p}{'seconds'};
			$gstats{$g}{'members'}++;
		}
		else {
			$gstats{$g}{'groupinfo'} = 
			$stats{$u}{'groupinfo'};
			
			$gstats{$g}{$d}{$p}{'files'} = 
			$stats{$u}{$d}{$p}{'files'};
			
			$gstats{$g}{$d}{$p}{'bytes'} = 
			$stats{$u}{$d}{$p}{'bytes'};
			
			$gstats{$g}{$d}{$p}{'seconds'} = 
			$stats{$u}{$d}{$p}{'seconds'};
			$gstats{$g}{'members'} = 1;
		}
	}
   	
	foreach my $g (sort {$gstats{$b}{$d}{$p}{'bytes'} <=> 
	                     $gstats{$a}{$d}{$p}{'bytes'}} 
			     keys %gstats) 
	{
		my $mbsize = $gstats{$g}{$d}{$p}{'bytes'} / 0x100000;
		my $speed  = $gstats{$g}{$d}{$p}{'seconds'} > 0 ? 
		             $gstats{$g}{$d}{$p}{'bytes'} / 
			     ($gstats{$g}{$d}{$p}{'seconds'} * 1024) : 0;
		my $files  = $gstats{$g}{$d}{$p}{'files'};
		my $info   = $gstats{$g}{'groupinfo'};
		my $members = $gstats{$g}{'members'};
		$rank++;
		if (($rank > $lines) or ($mbsize == 0)) { last; }
		my $fmt = $rank % 2 ? $gbody1 : $gbody2;
		$bodydata .= sprintf ($fmt, $rank, 
		                      $g . '.' x 9, 
				      $members, 
		                      $info . '.' x 39,  
		                      $files, $mbsize, $speed);
	}
	return $bodydata;
}


# fix me !!! this should call an internal function instead of this
# signal
sub refresh_stats {
	open (PIDFILE, "< $FTP::cfg_var_path/ftpd.pid") or 
		die "ERROR unable to open pid file";
	my $pid = <PIDFILE>;
	chomp($pid);
	kill USR1 => $pid;
	sleep(1);
}


sub print_stats {
	my $args = shift;
	my $type = shift;
	my $direction = shift;
	my $period = shift;

	my $section = $FTP::var_section;
	my $bodydata;
	my $header;
	my $footer;
	my $lines = 10;
	
        # IF YOU WANT A FASTER EXECUTION REMOVE THIS LINE
	# AND LET CRONTAB SEND THE SIGUSR TO THE FTPD
	refresh_stats();

	# -l <lines>
	if ($args =~ /-l (\d+)/i) {
		$lines = $1;	
	}

	# -S <section>
	if ($args =~ /-s (\w+)/i) {
		$section = $1;	
	}

	if (not $section) {
		$section = 'global';
	}
	elsif (($show_global) and ($section eq 'default')) {
		$section = 'global';
	}

	# load stats from file to hash
	my %stats = load_stats($section);

	if ($type eq "user") {
		$header = $uheader;
		$bodydata = print_user_stats(\%stats, $direction, $period, 
		                             $lines);
	}
	if ($type eq "group") {
		$header = $gheader;
		$bodydata = print_group_stats(\%stats, $direction, $period, 
		                              $lines);
	}

	# replace some vars in the header
	$header =~ s/\$SECTION/$section/;
	$header =~ s/\$PERIOD/$period/;

	if ($type eq "user") {
	    $footer = $ufooter;
	} else {
	    $footer = $gfooter;
	}
	return (200, $header . $bodydata . $footer);
}


############################################################################
# specialized functions for SITE WKUP ...                                  #
############################################################################

# week
my $show_user_upload_week_stats = sub {
	my $args = shift;
	print_stats($args, 'user', 'upload', 'week');
};
my $show_user_download_week_stats = sub {
	my $args = shift;
	print_stats($args, 'user', 'download', 'week');
};

my $show_group_upload_week_stats = sub {
	my $args = shift;
	print_stats($args, 'group', 'upload', 'week');
};
my $show_group_download_week_stats = sub {
	my $args = shift;
	print_stats($args, 'group', 'download', 'week');
};

#day
my $show_user_upload_day_stats = sub {
	my $args = shift;
	print_stats($args, 'user', 'upload', 'day');
};
my $show_user_download_day_stats = sub {
	my $args = shift;
	print_stats($args, 'user', 'download', 'day');
};

my $show_group_upload_day_stats = sub {
	my $args = shift;
	print_stats($args, 'group', 'upload', 'day');
};
my $show_group_download_day_stats = sub {
	my $args = shift;
	print_stats($args, 'group', 'download', 'day');
};

# month
my $show_user_upload_month_stats = sub {
	my $args = shift;
	print_stats($args, 'user', 'upload', 'month');
};
my $show_user_download_month_stats = sub {
	my $args = shift;
	print_stats($args, 'user', 'download', 'month');
};

my $show_group_upload_month_stats = sub {
	my $args = shift;
	print_stats($args, 'group', 'upload', 'month');
};
my $show_group_download_month_stats = sub {
	my $args = shift;
	print_stats($args, 'group', 'download', 'month');
};

# year
my $show_user_upload_year_stats = sub {
	my $args = shift;
	print_stats($args, 'user', 'upload', 'year');
};
my $show_user_download_year_stats = sub {
	my $args = shift;
	print_stats($args, 'user', 'download', 'year');
};

my $show_group_upload_year_stats = sub {
	my $args = shift;
	print_stats($args, 'group', 'upload', 'year');
};
my $show_group_download_year_stats = sub {
	my $args = shift;
	print_stats($args, 'group', 'download', 'year');
};

# total
my $show_user_upload_total_stats = sub {
	my $args = shift;
	print_stats($args, 'user', 'upload', 'total');
};
my $show_user_download_total_stats = sub {
	my $args = shift;
	print_stats($args, 'user', 'download', 'total');
};

my $show_group_upload_total_stats = sub {
	my $args = shift;
	print_stats($args, 'group', 'upload', 'total');
};

my $show_group_download_total_stats = sub {
	my $args = shift;
	print_stats($args, 'group', 'download', 'total');
};


############################################################################
# generic function for SITE TOP                                            #
############################################################################

my $show_stats = sub {
	my $args = shift;
	if ($args =~ /(user|group) +(download|upload) +(day|week|month|year|total)/) {
		return print_stats($args, $1, $2, $3);
	}
	if ($args =~ /(u|g)(d|u)(d|w|m|y|t)/) {
		my ($type, $direction, $period);
		if ($1 eq 'u') { $type = 'user'; }
		else { $type = 'group'; }
		if ($2 eq 'u') { $direction = 'upload'; }
		else { $direction = 'download'; }
		if ($3 eq 'd') { $period = 'day'; }
		elsif ($3 eq 'w') { $period = 'week'; }
		elsif ($3 eq 'm') { $period = 'month'; }
		elsif ($3 eq 'y') { $period = 'year'; }
		else { $period = 'total'; }
		return print_stats($args, $type, $direction, $period);
	}
	else {
		# fixme! use the help system instead
		return (200, "\nSyntax:\n" .
		             "  SITE TOP <user|group> <download|upload> <day|week|month|year|total>\n" .
		             "  SITE TOP <u|g><d|u><d|w|m|y|t>\n" .
		             "Example:\n" .
		             "  SITE TOP user upload week\n" .
		             "  SITE TOP uuw\n\n");
	}
};


# shortcuts to call script from msg files
sub dayup {
	&$show_user_upload_day_stats(@_);
}
sub daydn {
	&$show_user_download_day_stats(@_);
}
sub wkup {
	&$show_user_upload_week_stats(@_);
}
sub wkdn {
	&$show_user_download_week_stats(@_);
}
sub monthup { 
	&$show_user_upload_month_stats(@_);
}
sub monthdn {
	&$show_user_download_month_stats(@_);
}
sub yearup { 
	&$show_user_upload_year_stats(@_);
}
sub yeardn {
	&$show_user_download_year_stats(@_);
}
sub alup {
	&$show_user_upload_total_stats(@_);
}
sub aldn {
	&$show_user_download_total_stats(@_);
}

sub gdayup {
	&$show_group_upload_day_stats(@_);
}
sub gdaydn {
	&$show_group_download_day_stats(@_);
}
sub gpwk { 
	&$show_group_upload_week_stats(@_);
}
sub gwkup { 
	&$show_group_upload_week_stats(@_);
}
sub gwkdn {
	&$show_group_download_week_stats(@_);
}
sub gpwd {
	&$show_group_download_week_stats(@_);
}	
sub galup {
	&$show_group_upload_total_stats(@_);
}
sub gpal {
	&$show_group_upload_total_stats(@_);
}
sub galdn {
	&$show_group_download_total_stats(@_);
}
sub gpad {
	&$show_group_download_total_stats(@_);
}

# bind commands
&FTP::bind_site_cmd ("top", $show_stats);
&FTP::bind_site_cmd ("tops", $show_stats);

# only the most common ones, maybe we should add all here !?
&FTP::bind_site_cmd ("dayup", $show_user_upload_day_stats);
&FTP::bind_site_cmd ("daydn", $show_user_download_day_stats);
&FTP::bind_site_cmd ("wkup", $show_user_upload_week_stats);
&FTP::bind_site_cmd ("wkdn", $show_user_download_week_stats);
&FTP::bind_site_cmd ("monthup", $show_user_upload_month_stats);
&FTP::bind_site_cmd ("monthdn", $show_user_download_month_stats);
&FTP::bind_site_cmd ("yearup", $show_user_upload_year_stats);
&FTP::bind_site_cmd ("yeardn", $show_user_download_year_stats);
&FTP::bind_site_cmd ("alup", $show_user_upload_total_stats);
&FTP::bind_site_cmd ("aldn", $show_user_download_total_stats);

&FTP::bind_site_cmd ("gdayup", $show_group_upload_day_stats);
&FTP::bind_site_cmd ("gdaydn", $show_group_download_day_stats);
&FTP::bind_site_cmd ("gwkup", $show_group_upload_week_stats);
&FTP::bind_site_cmd ("gpwk", $show_group_upload_week_stats);
&FTP::bind_site_cmd ("gpwd", $show_group_download_week_stats);
&FTP::bind_site_cmd ("gwkdn", $show_group_download_week_stats);
&FTP::bind_site_cmd ("gmup", $show_group_upload_month_stats);
&FTP::bind_site_cmd ("gmdn", $show_group_download_month_stats);
&FTP::bind_site_cmd ("gyup", $show_group_upload_year_stats);
&FTP::bind_site_cmd ("gydn", $show_group_download_year_stats);
&FTP::bind_site_cmd ("galup", $show_group_upload_total_stats);
&FTP::bind_site_cmd ("gpal", $show_group_upload_total_stats);
&FTP::bind_site_cmd ("galdn", $show_group_download_total_stats);
&FTP::bind_site_cmd ("gpad", $show_group_download_total_stats);

# --- end of file ---